"""
ArmorIDs - autogenerated from decompiled Terraria .cs files
Terraria namespace: Terraria.ID
Terraria version: v1.4.5.5
Generator: TerrariaPyGen v1.1.1 (Maksim Stashkevich)
Generated on: 2026-02-15 09:24
"""

from terrex.id.set_factory import SetFactory
from enum import IntEnum, auto


class ArmorIDs:
    Count = 293

    class Back(IntEnum):
        BeeCloak = 1
        StarCloak = 2
        CrimsonCloak = 3
        MysteriousCape = 4
        RedCape = 5
        WinterCape = 6
        MagicQuiver = 7
        ArchitectGizmoPack = 8
        HivePack = 9
        AnglerTackleBag = 10
        ApprenticeDark = 11
        RedRidingHuntress = 12
        ShinobiInfiltrator = 13
        ManaCloak = 14
        MoltenQuiver = 15
        StalkersQuiver = 16
        ClothiersJacket = 17
        SpaceCreatureShirt = 18
        FoxShirt = 19
        VampireShirt = 20
        CatShirt = 21
        SuperHeroCostumeMale = 22
        SuperHeroCostumeFemale = 23
        HunterCloak = 24
        DogTail = 25
        FoxTail = 26
        LizardTail = 27
        BunnyTail = 28
        HallowedCape = 29
        PlaguebringerCloak = 30
        RoninCloak = 31
        FloretProtecterChestplate = 32
        LavaproofTackleBag = 33
        PrinceCape = 34
        HandOfCreation = 35
        ShimmerCloak = 36
        ChickenBonesRobe = 37
        ChippysWings = 38
        LunasCloak = 39
        Count = 40

    class Balloon(IntEnum):
        BlizzardinaBalloon = 1
        BlueHorseshoeBalloon = 2
        BundleofBalloons = 3
        CloudinaBalloon = 4
        FartinaBalloon = 5
        SandstorminaBalloon = 6
        HoneyBalloon = 7
        ShinyRedBalloon = 8
        WhiteHorseshoeBalloon = 9
        YellowHorseshoeBalloon = 10
        BalloonPufferfish = 11
        SharkronBalloon = 12
        GreenHorseshoeBalloon = 13
        AmberHorseshoeBalloon = 14
        PinkHorseshoeBalloon = 15
        BundledPartyBalloons = 16
        BalloonAnimal = 17
        RoyalScepter = 18
        HorseshoeBundle = 19
        Count = 20

    class Beard(IntEnum):
        GingerBeard = 1
        WilsonBeardShort = 2
        WilsonBeardLong = 3
        WilsonBeardMagnificent = 4
        Count = 5

    class Body(IntEnum):
        FamiliarShirt = 0
        CopperChainmail = 1
        IronChainmail = 2
        SilverChainmail = 3
        GoldChainmail = 4
        ShadowScalemail = 5
        MeteorSuit = 6
        NecroBreastplate = 7
        JungleShirt = 8
        MoltenBreastplate = 9
        TuxedoShirt = 10
        PlumbersShirt = 11
        HerosShirt = 12
        ArchaeologistsJacket = 13
        NinjaShirt = 14
        Robe = 15
        TheDoctorsShirt = 16
        CobaltBreastplate = 17
        MythrilChainmail = 18
        AdamantiteBreastplate = 19
        MiningShirt = 20
        Werewolf = 21
        Merman = 22
        ClownShirt = 23
        HallowedPlateMail = 24
        SantaShirt = 25
        RedsBreastplate = 26
        FrostBreastplate = 27
        TinChainmail = 28
        LeadChainmail = 29
        TungstenChainmail = 30
        PlatinumChainmail = 31
        WoodBreastplate = 32
        EbonwoodBreastplate = 33
        RichMahoganyBreastplate = 34
        PearlwoodBreastplate = 35
        RuneRobe = 36
        CrimsonScalemail = 37
        EskimoCoat = 38
        SteampunkShirt = 39
        BeeShirt = 40
        PrincessCostume = 41
        PharaohsRobe = 42
        MummyShirt = 43
        CowboyJacket = 44
        PirateShirt = 45
        CactusBreastplate = 46
        ShadewoodBreastplate = 47
        AncientShadowScalemail = 48
        AncientCobaltBreastplate = 49
        PinkEskimoCoat = 50
        ChlorophytePlateMail = 51
        RainCoat = 52
        TikiShirt = 53
        PalladiumBreastplate = 54
        OrichalcumBreastplate = 55
        TitaniumBreastplate = 56
        SailorShirt = 57
        AmethystRobe = 58
        TopazRobe = 59
        SapphireRobe = 60
        EmeraldRobe = 61
        RubyRobe = 62
        DiamondRobe = 63
        WhiteTuxedoShirt = 64
        TurtleScaleMail = 65
        SpectreRobe = 66
        ShroomiteBreastplate = 67
        CenxsBreastplate = 68
        CrownosBreastplate = 69
        WillsBreastplate = 70
        JimsBreastplate = 71
        AaronsBreastplate = 72
        DTownsBreastplate = 73
        CenxsDress = 74
        PumpkinBreastplate = 75
        NurseShirt = 76
        DyeTraderRobe = 77
        CyborgShirt = 78
        CreeperShirt = 79
        CatShirt = 80
        GhostShirt = 81
        PumpkinShirt = 82
        RobotShirt = 83
        UnicornShirt = 84
        VampireShirt = 85
        LeprechaunShirt = 86
        PixieShirt = 87
        PrincessDress = 88
        WitchDress = 89
        BrideofFrankensteinDress = 90
        KarateTortoiseShirt = 91
        ScarecrowShirt = 92
        ReaperRobe = 93
        FoxShirt = 94
        SpookyBreastplate = 95
        SpaceCreatureShirt = 96
        WolfShirt = 97
        TreasureHunterShirt = 98
        DryadCoverings = 99
        MrsClausShirt = 100
        TreeShirt = 101
        ParkaCoat = 102
        UglySweater = 103
        ElfShirt = 104
        BeetleScaleMail = 105
        BeetleShell = 106
        Gi = 165
        Kimono = 166
        GypsyRobe = 167
        BeeBreastplate = 168
        AnglerVest = 169
        SpiderBreastplate = 170
        MermaidAdornment = 171
        FishCostumeShirt = 172
        BorealWoodBreastplate = 173
        PalmWoodBreastplate = 174
        VortexBreastplate = 175
        NebulaBreastplate = 176
        SolarFlareBreastplate = 177
        MartianCostumeShirt = 178
        MartianUniformTorso = 179
        SolarCultistRobe = 180
        LunarCultistRobe = 181
        GladiatorBreastplate = 182
        LazuresValkyrieCloak = 183
        TaxCollectorsSuit = 184
        ClothiersJacket = 185
        BuccaneerTunic = 186
        ObsidianLongcoat = 187
        FallenTuxedoShirt = 188
        FossilPlate = 189
        StardustPlate = 190
        WeddingDress = 191
        Yoraiz0rsUniform = 192
        SkiphssSkin = 193
        LokisBreastplate = 194
        SillySunflowerTops = 195
        PedguinsJacket = 196
        AncientArmor = 197
        AncientBattleArmor = 198
        Lamia = 199
        ApprenticeRobe = 200
        SquirePlating = 201
        HuntressJerkin = 202
        MonkShirt = 203
        ValhallaKnight = 204
        ApprenticeDark = 205
        RedRidingHuntress = 206
        ShinobiInfiltrator = 207
        ArkhalisShirt = 208
        LeinforsShirt = 209
        Maid = 210
        MaidAlt = 211
        Golf = 212
        AmberRobe = 213
        GameMasterShirt = 214
        StarPrincessDress = 215
        ChefOutfit = 216
        SuperHeroCostume = 217
        PrettyPinkDress = 218
        UndertakerCoat = 219
        FuneralCoat = 220
        VictorianGothDress = 221
        GhostarShirt = 222
        DrManFlyLabCoat = 223
        ButcherApron = 224
        SafemanSunDress = 225
        FoodBarbarianArmor = 226
        GroxTheGreatArmor = 227
        MushroomVest = 228
        AncientHallowedPlateMail = 229
        CrystalNinjaChestplate = 230
        GraduationGownBlue = 231
        GraduationGownMaroon = 232
        GraduationGownBlack = 233
        DeadMansSweater = 234
        PlaguebringerChestplate = 235
        RoninShirt = 236
        TimelessTravelerRobe = 237
        FloretProtectorChestplate = 238
        CapricornChestplate = 239
        TVHeadSuit = 240
        FlinxFurCoat = 241
        RoyalDressTop = 242
        PrinceUniform = 243
        WilsonShirt = 244
        WillowShirt = 245
        AshWoodBreastplate = 246
        LincolnsHoodie = 247
        DeadCellsBeheadedBody = 248
        MoonLordBody = 249
        ChickenBonesBody = 250
        ChickenBonesRobe = 251
        UpgradedMiningBody = 252
        UpgradedFishingBody = 253
        ChippysBody = 254
        BlueBikiniBody = 255
        RedSwimsuitBody = 256
        HeroicisBody = 257
        LilacDuskBody = 258
        KazzymodusChestpiece = 259
        PalworldPalMetalArmorBody = 260
        LunasBody = 261
        Count = 262

    class Face(IntEnum):
        NaturesGift = 1
        ArcticDivingGear = 2
        JellyfishDivingGear = 3
        DivingGear = 4
        Blindfold = 5
        ObsidianRose = 6
        AngelHalo = 7
        JungleRose = 8
        ArcaneFlower = 9
        LavaSkull = 10
        MoltenSkullRose = 11
        ObsidianSkull = 12
        ObsidianSkullRose = 13
        SpectreGoggles = 14
        LavaSkullAlt = 15
        MoltenSkullRoseAlt = 16
        ObsidianSkullAlt = 17
        ObsidianSkullRoseAlt = 18
        BoneHelm = 19
        ReflectiveShades = 20
        JimsDroneVisor = 21
        WeldingMask = 22
        ChippysHeadband = 23
        Count = 24

    class Front(IntEnum):
        CrimsonCloak = 1
        MysteriousCape = 2
        RedCape = 3
        WinterCape = 4
        ManaCloak = 5
        TaxCollectorsSuit = 6
        VampireShirt = 7
        HunterCloak = 8
        PlaguebringerCloak = 9
        RoninCloak = 10
        PrinceCape = 11
        ShimmerCloak = 12
        DeadCellsBeheadedBody = 13
        ChickenBonesRobe = 14
        ChippysWings = 15
        LunasCloak = 16
        Count = 17

    class HandOff(IntEnum):
        FireGauntlet = 1
        HandWarmer = 2
        MagicCuffs = 3
        MechanicalGlove = 4
        PowerGlove = 5
        ClimbingClaws = 6
        MasterNinjaGear = 6
        TigerClimbingGear = 6
        Shackle = 7
        TitanGlove = 8
        FeralClaws = 9
        CelestialCuffs = 10
        YoyoGlove = 11
        BersekerGlove = 12
        FrogWebbing = 13
        BoneGlove = 14
        HandOfCreation = 15
        Count = 16

    class HandOn(IntEnum):
        ManaRegenerationBand = 1
        BandofRegeneration = 2
        BandofStarpower = 3
        CharmofMyths = 4
        FeralClaws = 5
        FireGauntlet = 6
        HandWarmer = 7
        MagicCuffs = 8
        MechanicalGlove = 9
        PowerGlove = 10
        ClimbingClaws = 11
        MasterNinjaGear = 11
        TigerClimbingGear = 11
        Shackle = 12
        SunStone = 13
        MoonStone = 14
        TitanGlove = 15
        DiamondRing = 16
        CelestialCuffs = 17
        YoyoGlove = 18
        HuntressBuckler = 19
        BersekerGlove = 20
        FrogWebbing = 21
        BoneGlove = 22
        HandOfCreation = 23
        LavaCharm = 24
        Count = 25

    class Head(IntEnum):
        FamiliarWig = 0
        CopperHelmet = 1
        IronHelmet = 2
        SilverHelmet = 3
        GoldHelmet = 4
        ShadowHelmet = 5
        MeteorHelmet = 6
        NecroHelmet = 7
        JungleHat = 8
        MoltenHelmet = 9
        Goggles = 10
        MiningHelmet = 11
        Sunglasses = 12
        EmptyBucket = 13
        WizardHat = 14
        TopHat = 15
        SummerHat = 16
        BunnyHood = 17
        PlumbersHat = 18
        HerosHat = 19
        FishBowl = 20
        ArchaeologistsHat = 21
        NinjaHood = 22
        Unused = 23
        RedHat = 24
        RobotHat = 25
        GoldCrown = 26
        DivingHelmet = 27
        MimeMask = 28
        CobaltHat = 29
        CobaltHelmet = 30
        CobaltMask = 31
        MythrilHood = 32
        MythrilHelmet = 33
        MythrilHat = 34
        AdamantiteHeadgear = 35
        AdamantiteHelmet = 36
        AdamantiteMask = 37
        Werewolf = 38
        Merman = 39
        ClownHat = 40
        HallowedHelmet = 41
        HallowedHeadgear = 42
        HallowedMask = 43
        SantaHat = 44
        RedsHelmet = 45
        FrostHelmet = 46
        TinHelmet = 47
        LeadHelmet = 48
        TungstenHelmet = 49
        PlatinumHelmet = 50
        PlatinumCrown = 51
        WoodHelmet = 52
        EbonwoodHelmet = 53
        RichMahoganyHelmet = 54
        PearlwoodHelmet = 55
        RuneHat = 56
        CrimsonHelmet = 57
        EskimoHood = 58
        SteampunkHat = 59
        BeeHat = 60
        PharaohsMask = 61
        Tiara = 62
        GreenCap = 63
        MushroomCap = 64
        TamOShanter = 65
        MummyMask = 66
        CowboyHat = 67
        PirateHat = 68
        VikingHelmet = 69
        CactusHelmet = 70
        ShadewoodHelmet = 71
        AncientIronHelmet = 72
        AncientGoldHelmet = 73
        AncientShadowHelmet = 74
        AncientNecroHelmet = 75
        AncientCobaltHelmet = 76
        PinkEskimoHood = 77
        ChlorophyteMask = 78
        ChlorophyteHelmet = 79
        ChlorophyteHeadgear = 80
        RainHat = 81
        TikiMask = 82
        PalladiumMask = 83
        PalladiumHelmet = 84
        PalladiumHeadgear = 85
        OrichalcumMask = 86
        OrichalcumHelmet = 87
        OrichalcumHeadgear = 88
        TitaniumMask = 89
        TitaniumHelmet = 90
        TitaniumHeadgear = 91
        UmbrellaHat = 92
        Skull = 93
        BallaHat = 94
        GangstaHat = 95
        SailorHat = 96
        EyePatch = 97
        SkeletronMask = 98
        TurtleHelmet = 99
        Beanie = 100
        SpectreHood = 101
        SWATHelmet = 102
        ShroomiteHeadgear = 103
        ShroomiteMask = 104
        ShroomiteHelmet = 105
        CenxsTiara = 106
        CrownosMask = 107
        WillsHelmet = 108
        JimsHelmet = 109
        AaronsHelmet = 110
        DTownsHelmet = 111
        PumpkinHelmet = 112
        NurseHat = 113
        WizardsHat = 114
        GuyFawkesMask = 115
        SteampunkGoggles = 116
        CyborgHelmet = 117
        CreeperMask = 118
        CatMask = 119
        GhostMask = 120
        PumpkinMask = 121
        RobotMask = 122
        UnicornMask = 123
        VampireMask = 124
        WitchHat = 125
        LeprechaunHat = 126
        PrincessHat = 127
        BrideofFrankensteinMask = 128
        KarateTortoiseMask = 129
        ScarecrowHat = 130
        ReaperHood = 131
        FoxMask = 132
        CatEars = 133
        SpookyHelmet = 134
        SpaceCreatureMask = 135
        WolfMask = 136
        JackOLanternMask = 137
        GiantBow = 138
        ReindeerAntlers = 139
        MrsClausHat = 140
        TreeMask = 141
        ParkaHood = 142
        SnowHat = 143
        ElfHat = 144
        Fez = 145
        BrainofCthulhuMask = 146
        WallofFleshMask = 147
        TwinMask = 148
        SkeletronPrimeMask = 149
        QueenBeeMask = 150
        PlanteraMask = 151
        GolemMask = 152
        EaterofWorldsMask = 153
        EyeofCthulhuMask = 154
        DestroyerMask = 155
        SpectreMask = 156
        BeetleHelmet = 157
        PeddlersHat = 158
        MagicHat = 159
        BeeHeadgear = 160
        AnglerHat = 161
        SpiderMask = 162
        SeashellHairpin = 163
        KingSlimeMask = 164
        FishCostumeMask = 165
        BorealWoodHelmet = 166
        PalmWoodHelmet = 167
        DukeFishronMask = 168
        VortexHelmet = 169
        NebulaHelmet = 170
        SolarFlareHelmet = 171
        MoonMask = 172
        SunMask = 173
        MartianCostumeMask = 174
        MartianUniformHelmet = 175
        SolarCultistHood = 176
        LunarCultistHood = 177
        HiTekSunglasses = 178
        NightVisionHelmet = 179
        GladiatorHelmet = 180
        LazuresValkyrieCirclet = 181
        TaxCollectorsHat = 182
        DyeTradersTurban = 183
        BuccaneerBandana = 184
        ObsidianOutlawHat = 185
        AncientCultistMask = 186
        MoonLordMask = 187
        FossilHelmet = 188
        StardustHelmet = 189
        WeddingVeil = 190
        Yoraiz0rsRecoloredGoggles = 191
        SkiphssMask = 192
        LokisHelmet = 193
        EngineeringHelmet = 194
        PartyHat = 195
        SillySunflowerPetals = 196
        PedguinsHood = 197
        _0x33sAviators = 198
        AncientArmor = 199
        AncientBattleArmor = 200
        LamiaMale = 201
        LamiaFemale = 202
        ApprenticeHat = 203
        SquireGreatHelm = 204
        HuntressWig = 205
        MonkBrows = 206
        MaskBetsy = 207
        MaskDarkMage = 208
        MaskOgre = 209
        ValhallaKnight = 210
        ApprenticeDark = 211
        RedRidingHuntress = 212
        ShinobiInfiltrator = 213
        ArkhalisHat = 214
        LeinforsHat = 215
        UltraBrightHelmet = 216
        Maid = 217
        MaidAlt = 218
        GolfHat = 219
        GolfVisor = 220
        GoldGoldfishBowl = 221
        StarPrincessCrown = 222
        DemonHorns = 223
        BunnyEars = 224
        DevilHorns = 225
        Fedora = 226
        ChefHat = 227
        StarHairpin = 228
        HeartHairpin = 229
        SuperHeroMask = 230
        UndertakerHat = 231
        PrettyPinkRibbon = 232
        FuneralHat = 233
        VictorianGothHat = 234
        GhostarSkullPin = 235
        DrManFlyMask = 236
        ButcherMask = 237
        SafemanSunHair = 238
        FoodBarbarianHelm = 239
        GroxTheGreatHelm = 240
        RockGolemHead = 241
        DogEars = 242
        FoxEars = 243
        LizardEars = 244
        PandaEars = 245
        DogEarsBack = 246
        FoxEarsBack = 247
        LizardEarsBack = 248
        PandaEarsBack = 249
        MushroomHat = 250
        FairyQueenMask = 251
        CatEarsBack = 252
        BunnyEarsBack = 253
        HallowedHood = 254
        AncientHallowedMask = 255
        AncientHallowedHelmet = 256
        AncientHallowedHeadgear = 257
        AncientHallowedHood = 258
        RabbitOrder = 259
        QueenSlimeMask = 260
        CrystalNinjaHelmet = 261
        GraduationCapBlue = 262
        GraduationCapMaroon = 263
        GraduationCapBlack = 264
        BadgersHat = 265
        PlaguebringerHelmet = 266
        RoninHat = 267
        TimelessTravelerHood = 268
        FloretProtectorHelmet = 269
        CapricornMask = 270
        TVHeadMask = 271
        RoyalTiara = 272
        GarlandHat = 273
        VulkelfEars = 274
        Eyebrella = 275
        DeerclopsMask = 276
        GoblorcEars = 277
        AshWoodHelmet = 278
        JimsCap = 279
        DizzyHat = 280
        LincolnsHood = 281
        DeadCellsBeheadedHead = 282
        ChlorophyteVisor = 283
        ChickenBonesHead = 284
        UpgradedMiningHead = 285
        UpgradedFishingHead = 286
        ChippysHead = 287
        HeroicisHead = 288
        HallowedCrown = 289
        LilacDuskHead = 290
        KazzymodusHood = 291
        LunasHead = 292
        Count = 293

    class Legs(IntEnum):
        FamiliarPants = 0
        CopperGreaves = 1
        IronGreaves = 2
        SilverGreaves = 3
        GoldGreaves = 4
        ShadowGreaves = 5
        MeteorLeggings = 6
        NecroGreaves = 7
        JunglePants = 8
        MoltenGreaves = 9
        TuxedoPants = 10
        PlumbersPants = 11
        HerosPants = 12
        ArchaeologistsPants = 13
        NinjaPants = 14
        TheDoctorsPants = 15
        CobaltLeggings = 16
        MythrilGreaves = 17
        AdamantiteLeggings = 18
        MiningPants = 19
        Werewolf = 20
        Merman = 21
        ClownPants = 22
        HallowedGreaves = 23
        SantaPants = 24
        RedsLeggings = 25
        FrostLeggings = 26
        TinGreaves = 27
        LeadGreaves = 28
        TungstenGreaves = 29
        PlatinumGreaves = 30
        WoodGreaves = 31
        EbonwoodGreaves = 32
        RichMahoganyGreaves = 33
        PearlwoodGreaves = 34
        CrimsonGreaves = 35
        EskimoPants = 36
        SteampunkPants = 37
        BeePants = 38
        MummyPants = 39
        CowboyPants = 40
        PiratePants = 41
        CactusLeggings = 42
        ShadewoodGreaves = 43
        AncientShadowGreaves = 44
        AncientCobaltLeggings = 45
        PinkEskimoPants = 46
        ChlorophyteGreaves = 47
        TikiPants = 48
        PalladiumLeggings = 49
        OrichalcumLeggings = 50
        TitaniumLeggings = 51
        SailorPants = 52
        WhiteTuxedoPants = 53
        TurtleLeggings = 54
        SpectrePants = 55
        ShroomiteLeggings = 56
        CenxsLeggings = 57
        CrownosLeggings = 58
        WillsLeggings = 59
        JimsLeggings = 60
        AaronsLeggings = 61
        DTownsLeggings = 62
        CenxsDressPants = 63
        PumpkinLeggings = 64
        NursePants = 65
        CyborgPants = 66
        CreeperPants = 67
        CatPants = 68
        PumpkinPants = 69
        RobotPants = 70
        UnicornPants = 71
        VampirePants = 72
        LeprechaunPants = 73
        PixiePants = 74
        WitchBoots = 75
        KarateTortoisePants = 76
        ScarecrowPants = 77
        FoxPants = 78
        SpookyLeggings = 79
        SpaceCreaturePants = 80
        WolfPants = 81
        TreasureHunterPants = 82
        DryadLoincloth = 83
        MrsClausHeels = 84
        TreeTrunks = 85
        ParkaPants = 86
        ElfPants = 87
        BeetleLeggings = 98
        GiFemaleExtension = 99
        GypsyRobeMaleExtension = 101
        GypsyRobeFemaleExtension = 102
        BeeGreaves = 103
        AnglerPants = 104
        SpiderGreaves = 105
        MermaidTail = 106
        FishCostumeFinskirt = 107
        BorealWoodGreaves = 108
        PalmWoodGreaves = 109
        VortexLeggings = 110
        NebulaLeggings = 111
        SolarFlareLeggings = 112
        MartianCostumePants = 113
        MartianUniformPants = 114
        GiMaleExtension = 118
        GladiatorLeggings = 122
        TaxCollectorsPants = 124
        ClothiersPants = 125
        BuccaneerPantaloons = 126
        ObsidianPants = 127
        FallenTuxedoPants = 128
        FossilGreaves = 129
        StardustLeggings = 130
        Yoraiz0rsSkirt = 132
        SkiphssBearButt = 133
        LokisGreaves = 134
        SillySunflowerBottoms = 138
        PedguinsTrousers = 139
        DjinnsCurse = 140
        AncientArmor = 141
        AncientBattleArmor = 142
        Lamia = 143
        ApprenticeTrousers = 144
        SquireGreaves = 145
        HuntressPantsMale = 146
        HuntressPantsFemale = 147
        MonkPants = 148
        ApprenticeRobeExtension = 149
        SquirePlatingExtension = 150
        HuntressJerkinExtension = 151
        ValhallaKnight = 152
        ApprenticeDark = 153
        RedRidingHuntress = 154
        RedRidingHuntress_Male = 155
        ShinobiInfiltrator = 156
        ArkhalisPants_Male = 157
        ArkhalisPants_Female = 158
        LeinforsPants = 159
        LeinforsPantsCoat = 160
        ShinobiInfiltratorBodyExtension = 161
        ForbiddenArmorBodyExtension = 162
        GladiatorBreastplateBodyExtension = 163
        BeeBreastplateBodyExtension = 164
        ReaperRobe = 165
        BrideOfFrankensteinDress = 166
        WitchDress = 167
        PrincessDress = 168
        GhostShirt = 169
        DTownBreastplateBodyExtension = 170
        RaincoatMaleBodyExtension = 171
        RaincoatFemaleBodyExtension = 172
        ObsidianLongcoatBodyExtension = 173
        ApprenticeDarkBodyExtension = 174
        TikiShirtBodyExtensionMale = 175
        TikiShirtBodyExtensionFemale = 176
        MaidBodyExtensionFemale = 177
        MaidBodyExtensionMale = 178
        MaidFemale = 179
        MaidMale = 180
        MaidAltBodyExtensionFemale = 181
        MaidAltBodyExtensionMale = 182
        MaidAltFemale = 183
        MaidAltMale = 184
        Golf = 185
        WitchDressBodyExtension = 186
        AmberRobe = 187
        GameMasterPants = 188
        StarPrincessDress = 189
        ChefPants = 190
        SuperHeroTightsMale = 191
        SuperHeroTightsFemale = 192
        PrettyPinkDressPantsMale = 193
        PrettyPinkDressPantsFemale = 194
        PrettyPinkDressSkirt = 195
        UndertakerCoat = 196
        FuneralPantsMale = 197
        FuneralPantsFemale = 198
        VictorianGothDressExtension = 199
        GhostarShirtExtensionFemale = 200
        GhostarShirtExtensionMale = 201
        GhostarLegsFemale = 202
        GhostarLegsMale = 203
        DrManFlyLabCoatLegs = 204
        ButcherPants = 205
        SafemanSunDressExtension = 206
        SafemanDressLeggingsFemale = 207
        SafemanDressLeggingsMale = 208
        FoodBarbarianGreaves = 209
        GroxTheGreatGreaves = 210
        MushroomPants = 211
        AncientHallowedGreaves = 212
        CrystalNinjaLeggings = 213
        GraduationGownBlueExtension = 214
        GraduationGownMaroonExtension = 215
        GraduationGownBlackExtension = 216
        MoonLordLegs = 217
        PlaguebringerGreaves = 218
        RoninPantsMale = 219
        RoninPantsFemale = 220
        RoninShirtExtension = 221
        TimelessTravelerBottom = 222
        TimelessTravelerRobeExtension = 223
        FloretProtectorLegs = 224
        CapricornLegs = 225
        CapricornTail = 226
        TVHeadPants = 227
        RoyalDressBottom = 228
        FlinxFurCoatExtension = 229
        PrincePants = 230
        WilsonPants = 231
        WillowSkirtMale = 232
        WillowSkirtFemale = 233
        AshWoodGreaves = 234
        LincolnsPants = 235
        DeadCellsBeheadedLegsMale = 236
        ChickenBonesLegs = 237
        ChickenBonesRobeExtension = 238
        ChickenBonesRobeExtensionBack = 239
        UpgradedMiningLegs = 240
        UpgradedFishingLegs = 241
        ChippysLegs = 242
        BlueBikiniLegs = 243
        RedSwimsuitExtension = 244
        GreenSwimshorts = 245
        GraySwimshorts = 246
        HeroicisLegs = 247
        DeadCellsBeheadedLegsFemale = 248
        LilacDuskLegsMale = 249
        LilacDuskLegsFemale = 250
        KazzymodusLeggings = 251
        PalworldPalMetalArmorLegs = 252
        LunasLegs = 253
        Count = 254

    class Neck(IntEnum):
        JellyfishNecklace = 1
        CrossNecklace = 2
        PanicNecklace = 3
        PygmyNecklace = 4
        StarVeil = 5
        SweetheartNecklace = 6
        SharkToothNecklace = 7
        WormScarf = 8
        ApprenticeScarf = 9
        Stinger = 10
        Magiluminescence = 11
        MoltenCharm = 12
        Count = 13

    class RocketBoots(IntEnum):
        NoneValue = 0
        SimpleRocketBoots = 1
        SpectreBoots = 2
        FrostsparkBoots = 3
        TerrasparkBoots = 4
        HellfireTreads = 5
        RedsLeggings = 6

    class Shield(IntEnum):
        CobaltShield = 1
        PaladinsShield = 2
        ObsidianShield = 3
        AnkhShield = 4
        ShieldofCthulhu = 5
        SquireShield = 6
        Frozen = 7
        Hero = 8
        BouncingShield = 9
        Count = 10

    class Shoe(IntEnum):
        Flipper = 1
        WaterWalkingBoots = 2
        Tabi = 3
        ShoeSpikes = 4
        TigerClimbingGear = 4
        FlurryBoots = 5
        HermesBoots = 6
        IceSkates = 7
        LavaWaders = 8
        FrostsparkBoots = 9
        LightningBoots = 10
        ObsidianWaterWalkingBoots = 11
        RocketBoots = 12
        SpectreBoots = 13
        MasterNinjaGear = 14
        FrogLeg = 15
        FlowerBoots = 16
        SailfishBoots = 17
        AmphibianBoots = 18
        FairyBoots = 19
        FrogFlipper = 20
        SandBoots = 21
        FlameWakerBoots = 22
        HellfireTreads = 23
        TerrasparkBoots = 24
        GlassSlipperMale = 25
        GlassSlipperFemale = 26
        RollerSkates = 27
        RollerSkatesGreen = 28
        RollerSkatesWhite = 29
        RollerSkatesPink = 30
        Count = 31

    class Waist(IntEnum):
        CloudinaBottle = 1
        CopperWatch = 2
        GoldWatch = 3
        PlatinumWatch = 4
        Toolbelt = 5
        ManaFlower = 6
        SilverWatch = 7
        TinWatch = 8
        TungstenWatch = 9
        BlackBelt = 10
        MasterNinjaGear = 10
        TsunamiinaBottle = 11
        MonkBelt = 12
        BlizzardinaBottle = 13
        FartinaJar = 14
        SandstorminaBottle = 15
        TreasureMagnet = 16
        Count = 17

    class Wing(IntEnum):
        DemonWings = 1
        AngelWings = 2
        RedsWings = 3
        Jetpack = 4
        ButterflyWings = 5
        FairyWings = 6
        HarpyWings = 7
        BoneWings = 8
        FlameWings = 9
        FrozenWings = 10
        SpectreWings = 11
        SteampunkWings = 12
        LeafWings = 13
        BatWings = 14
        BeeWings = 15
        DTownsWings = 16
        WillsWings = 17
        CrownosWings = 18
        CenxsWings = 19
        TatteredFairyWings = 20
        SpookyWings = 21
        Hoverboard = 22
        FestiveWings = 23
        BeetleWings = 24
        FinWings = 25
        FishronWings = 26
        MothronWings = 27
        LazuresBarrierPlatform = 28
        SolarWings = 29
        VortexBooster = 30
        NebulaMantle = 31
        StardustWings = 32
        Yoraiz0rsSpell = 33
        JimsWings = 34
        SkiphssPaws = 35
        LokisWings = 36
        BetsyWings = 37
        ArkhalisWings = 38
        LeinforsWings = 39
        GhostarsWings = 40
        SafemanWings = 41
        FoodBarbarianWings = 42
        GroxTheGreatWings = 43
        RainbowWings = 44
        LongTrailRainbowWings = 45
        CreativeWings = 46
        ChickenBonesWings = 47
        ChippysWings = 48
        HeroicisWings = 49
        KazzymodusWings = 50
        LunasWings = 51
        Count = 52


class ArmorIDsSets:
    class Back:
        factory = SetFactory(ArmorIDs.Back.Count)
        DrawInBackpackLayer = factory.create_bool_set(False, 7, 8, 9, 10, 15, 16, 32, 33)
        DrawInTailLayer = factory.create_bool_set(False, 18, 19, 21, 25, 26, 27, 28)
        IsACape = factory.create_bool_set(False, 1, 2, 3, 4, 5, 6, 14, 24, 34, 36, 39)

    class Balloon:
        factory = SetFactory(ArmorIDs.Balloon.Count)
        DrawInFrontOfBackArmLayer = factory.create_bool_set(False, 18)
        UsesTorsoFraming = factory.create_bool_set(False, 18)

    class Beard:
        factory = SetFactory(ArmorIDs.Beard.Count)
        UseHairColor = factory.create_bool_set(False, 2, 3, 4)

    class Body:
        factory = SetFactory(ArmorIDs.Body.Count)
        NeedsToDrawArm = factory.create_bool_set(False, 200, 202, 201, 203, 195, 205, 207, 206, 228, 242, 243, 255, 256)
        IncludedCapeBack = factory.create_int_set(-1, 207, 13, 206, 12, 205, 11, 185, 17, 96, 18, 94, 19, 80, 21, 217, 22, 24, 29, 238, 32)
        IncludedCapeBackFemale = factory.create_int_set(-1, 207, 13, 206, 12, 205, 11, 185, 17, 96, 18, 94, 19, 80, 21, 217, 23, 24, 29, 238, 32)
        IncludedCapeFront = factory.create_int_set(-1, 184, 6, 248, 13)
        UsesNewFramingCode = factory.create_bool_set(False, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 91, 92, 93, 94, 95, 96, 97, 98, 99, 100, 101, 102, 103, 104, 105, 106, 165, 166, 167, 168, 169, 170, 171, 172, 173, 174, 175, 176, 177, 178, 179, 180, 181, 182, 183, 184, 185, 186, 187, 188, 189, 190, 191, 192, 193, 194, 195, 196, 197, 198, 199, 200, 201, 202, 203, 204, 205, 206, 207, 208, 209, 210, 211, 212, 213, 214, 215, 216, 217, 218, 219, 220, 221, 222, 223, 224, 225, 226, 227, 228, 229, 230, 231, 232, 233, 234, 235, 236, 237, 238, 239, 240, 241, 242, 243, 244, 245, 246, 247, 248, 249, 250, 251, 252, 253, 254, 255, 256, 257, 258, 259, 260, 261)
        HidesShouldersAsCoat = factory.create_bool_set(False, 251)
        showsShouldersWhileJumping = factory.create_bool_set(False, 177, 190, 95, 46, 73, 92, 24, 54, 55, 56, 65, 66, 67, 69, 70, 71, 75, 105, 106, 175, 176, 182, 183, 187, 194, 197, 198, 201, 204, 226, 227, 229, 239, 259)
        shouldersAreAlwaysInTheBack = factory.create_bool_set(False, 190)
        DisableHandOnAndOffAccDraw = factory.create_bool_set(False, 83)
        DisableBeltAccDraw = factory.create_bool_set(False, 83, 82)

    class Face:
        factory = SetFactory(ArmorIDs.Face.Count)
        PreventHairDraw = factory.create_bool_set(False, 2, 3, 4, 19)
        OverrideHelmet = factory.create_bool_set(False, 2, 3, 4, 19)
        DrawInFaceUnderHairLayer = factory.create_bool_set(False, 5)
        DrawInFaceMaskLayer = factory.create_bool_set(False, 22)
        DrawInFaceFlowerLayer = factory.create_bool_set(False, 1, 6, 9, 8)
        DrawInFaceHeadLayer = factory.create_bool_set(False, 12, 10, 13, 11)
        AltFaceHead = factory.create_int_set(-1, 12, 17, 10, 15, 13, 18, 11, 16)
        CanDrawOnVelociraptorMount = factory.create_bool_set(False, 1, 6, 9, 8, 19, 7, 20, 21, 23)

    class Front:
        factory = SetFactory(ArmorIDs.Front.Count)
        DrawsInNeckLayer = factory.create_bool_set(False, 6)
        DrawsInNeckLayerRegardlessOfPlayerFrame = factory.create_bool_set(False, 13)
        DontDrawIfWearingAScarfOrCape = factory.create_bool_set(False, 13)
        IsACape = factory.create_bool_set(False, 1, 2, 3, 4, 5, 8, 11, 12, 16)
        HidesCompositeShoulders = factory.create_bool_set(False, 8, 11, 15, 16)

    class HandOff:
        factory = SetFactory(ArmorIDs.HandOff.Count)
        UsesNewFramingCode = factory.create_bool_set(False, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15)

    class HandOn:
        factory = SetFactory(ArmorIDs.HandOn.Count)
        UsesNewFramingCode = factory.create_bool_set(False, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 16, 17, 18, 19, 20, 21, 22, 23, 24)
        UsesOldFramingTexturesForWalking = factory.create_bool_set(False, 12, 8, 17)

    class Head:
        factory = SetFactory(ArmorIDs.Head.Count)
        FrontToBackID = factory.create_int_set(-1, 242, 246, 243, 247, 244, 248, 245, 249, 133, 252, 224, 253)
        UseSkinColor = factory.create_bool_set(False, 274, 277)
        HidesHead = factory.create_bool_set(False, 38, 135, 269, 282, 288)
        PreventBeardDraw = factory.create_bool_set(False, 118, 119, 107, 129, 127, 125, 136, 162, 168, 210, 213, 28, 135, 31, 17, 261, 169, 186, 47, 22, 105, 102, 120, 137, 199, 207, 260, 132, 155, 179, 108, 153, 164, 172, 208, 241, 271, 48, 154, 188, 187, 209, 46, 123, 269, 185, 276)
        UseAltFaceHeadDraw = factory.create_bool_set(False, 3, 4, 6, 7, 9, 17, 22, 27, 28, 31, 32, 37, 38, 39, 40, 42, 43, 45, 46, 47, 48, 49, 50, 57, 58, 61, 66, 69, 70, 72, 74, 75, 76, 77, 78, 82, 83, 86, 89, 93, 98, 101, 103, 104, 105, 107, 108, 109, 110, 111, 115, 118, 120, 122, 123, 124, 127, 128, 129, 131, 132, 134, 135, 136, 137, 142, 146, 147, 148, 149, 150, 151, 152, 153, 154, 155, 156, 157, 162, 164, 165, 168, 169, 170, 171, 172, 173, 174, 175, 176, 177, 178, 179, 185, 186, 187, 188, 189, 192, 193, 194, 200, 201, 202, 204, 207, 208, 209, 210, 211, 213, 214, 230, 236, 237, 239, 240, 241, 251, 254, 255, 258, 260, 266, 268, 269, 270, 271, 261, 8, 12, 30, 35, 36, 73, 79, 80, 84, 85, 87, 88, 90, 91, 102, 112, 141, 160, 180, 196, 205, 257, 256, 291, 292)
        CanDrawOnVelociraptorMount = factory.create_bool_set(False, 212, 241, 274, 277, 5, 278, 34, 205, 41, 42, 30, 8, 84, 79, 33, 91, 90, 99, 35, 4, 256, 257, 36, 85, 80, 1, 112, 2, 215, 201, 202, 199, 194, 206, 262, 259, 234, 226, 238, 250, 280, 232, 225, 223, 224, 219, 220, 216, 263, 273, 264, 267, 18, 19, 24, 21, 26, 25, 16, 14, 12, 275, 65, 51, 56, 40, 29, 81, 94, 95, 62, 67, 44, 68, 121, 114, 116, 113, 92, 106, 100, 96, 97, 141, 143, 145, 139, 140, 144, 126, 130, 133, 159, 158, 161, 166, 183, 181, 178, 184, 182, 13, 64, 60, 59, 73, 63, 117, 138, 160, 10, 235, 197, 195, 203, 191, 228, 233, 227, 217, 218, 245, 243, 244, 242, 231, 229, 222, 265, 272, 279, 15, 163, 288, 289, 290)

    class Legs:
        factory = SetFactory(ArmorIDs.Legs.Count)
        IncompatibleWithFrogLeg = factory.create_bool_set(False, 106, 143, 217, 222, 226, 228, 138)
        TreatRobeExtensionLikeNormalLegsWhenSitting = factory.create_bool_set(False, 101, 102, 118, 99, 196)

    class Neck:
        factory = SetFactory(ArmorIDs.Neck.Count)
        IsAScarf = factory.create_bool_set(False, 8, 9)

    class Shoe:
        factory = SetFactory(ArmorIDs.Shoe.Count)
        MaleToFemaleID = factory.create_int_set(-1, 25, 26)
        IsARollerSkate = factory.create_bool_set(False, 27, 28, 29, 30)

    class Waist:
        factory = SetFactory(ArmorIDs.Waist.Count)
        UsesTorsoFraming = factory.create_bool_set(False, 5, 10, 10, 12)
        IsABelt = factory.create_bool_set(False, 5, 10, 10, 12)

    class Wing:
        factory = SetFactory(ArmorIDs.Wing.Count)
        AlwaysAnimated = factory.create_bool_set(False, 22, 28, 45, 34, 48, 39, 40, 44)



class BitsByte:
    def __init__(self, value: int = 0):
        self.value = value & 0xFF  # byte limit

    def __getitem__(self, index: int) -> bool:
        if not 0 <= index < 8:
            raise IndexError("Index must be 0-7")
        return bool((self.value >> index) & 1)

    def __setitem__(self, index: int, val: bool):
        if not 0 <= index < 8:
            raise IndexError("Index must be 0-7")
        if val:
            self.value |= 1 << index
        else:
            self.value &= ~(1 << index)

    def __int__(self) -> int:
        return self.value

    def __repr__(self):
        return f"BitsByte({self.value:08b})"



from terrex.id import MessageID
from terrex.net.streamer import Reader, Writer
from terrex.packet.base import SyncPacket


class SyncEquipment(SyncPacket):
    id = MessageID.SyncEquipment

    # 0 - 49 = Inventory
    # 50 - 53 = Money
    # 54 - 57 = Ammunition
    # 58 = Cursor
    # 59 - 61 = Armor
    # 62 - 67 = Acsesuars
    # 68 ???
    # 69 - 71 = Armor decorations
    # 72 - 77 = Acsesuars decorations
    # 78 ???
    # 79 - 87 = Dye (красители)
    # 88 ???
    # 89 - 93 MiscEquips (обычный питомец, светящийся питомец, вагонетка, ездовой питомец, крюк)
    # 94 - 98 = MiscDyes (красители животных)
    # 99 - 138 = Piggy bank
    # 499 = Trash

    # not tested
    # 139 - 178 = Safe
    # 180 - 219 = Defender's Forge
    # 220 - 259 = Void Vault
    def __init__(
        self,
        player_id: int = 0,
        slot_id: int = 0,
        stack: int = 0,
        prefix: int = 0,
        item_netid: int = 0,
    ):
        self.player_id = player_id
        self.slot_id = slot_id
        self.stack = stack
        self.prefix = prefix
        self.item_netid = item_netid

    def read(self, reader: Reader):
        self.player_id = reader.read_byte()
        self.slot_id = reader.read_short()
        self.stack = reader.read_short()
        self.prefix = reader.read_byte()
        self.item_netid = reader.read_ushort()

    def write(self, writer: Writer):
        writer.write_byte(self.player_id)
        writer.write_short(self.slot_id)
        writer.write_short(self.stack)
        writer.write_byte(self.prefix)
        writer.write_ushort(self.item_netid)



from terrex.id import MessageID
from terrex.net.streamer import Reader
from terrex.packet.base import ServerPacket


class InvasionProgressReport(ServerPacket):
    id = MessageID.InvasionProgressReport

    def __init__(self, progress: int = 0, max_progress: int = 0, icon: int = 0, wave: int = 0):
        self.progress = progress
        self.max_progress = max_progress
        self.icon = icon
        self.wave = wave

    def read(self, reader: Reader):
        self.progress = reader.read_int()
        self.max_progress = reader.read_int()
        self.icon = reader.read_byte()
        self.wave = reader.read_byte()



from terrex.id import MessageID
from terrex.net.streamer import Reader, Writer
from terrex.packet.base import ClientPacket


class SendPassword(ClientPacket):
    id = MessageID.SendPassword

    def __init__(self, password: str = ""):
        self.password = password

    def write(self, writer: Writer):
        writer.write_dotnet_string(self.password)

    def read(self, reader: Reader) -> None:
        self.password = reader.read_dotnet_string()



from .any_var import AnyVar


class IntVar(AnyVar):
    def __init__(self, default: int, *, min: int | None = None, max: int | None = None, **kw):
        self._min = min
        self._max = max
        super().__init__(default, **kw)

    def _validate(self, value):
        if not isinstance(value, int):
            raise TypeError(f"'{self.get_field_name()}' expects int")
        if self._min is not None and value < self._min:
            raise ValueError(f"'{self.get_field_name()}' < {self._min}")
        if self._max is not None and value > self._max:
            raise ValueError(f"'{self.get_field_name()}' > {self._max}")



from __future__ import annotations

from terrex.net.streamer import Reader, Writer


class PlayerDeathReason:
    HAS_KILLER = 0x01
    HAS_KILLING_NPC = 0x02
    HAS_PROJECTILE_IDX = 0x04
    HAS_TYPE_OF_DEATH = 0x08
    HAS_PROJECTILE_TYPE = 0x10
    HAS_ITEM_TYPE = 0x20
    HAS_ITEM_PREFIX = 0x40
    HAS_CUSTOM_REASON = 0x80

    def __init__(
        self,
        reason_flags: int = 0,
        killer_player_id: int | None = None,
        killing_npc_index: int | None = None,
        projectile_index: int | None = None,
        death_type: int | None = None,
        projectile_type: int | None = None,
        item_type: int | None = None,
        item_prefix: int | None = None,
        custom_reason: str | None = None,
    ):
        self.reason_flags = reason_flags
        self.killer_player_id = killer_player_id
        self.killing_npc_index = killing_npc_index
        self.projectile_index = projectile_index
        self.death_type = death_type
        self.projectile_type = projectile_type
        self.item_type = item_type
        self.item_prefix = item_prefix
        self.custom_reason = custom_reason

    @classmethod
    def read(cls, reader: Reader) -> PlayerDeathReason:
        reason_flags = reader.read_byte()
        killer_player_id = None
        if reason_flags & cls.HAS_KILLER:
            killer_player_id = reader.read_short()
        killing_npc_index = None
        if reason_flags & cls.HAS_KILLING_NPC:
            killing_npc_index = reader.read_short()
        projectile_index = None
        if reason_flags & cls.HAS_PROJECTILE_IDX:
            projectile_index = reader.read_short()
        death_type = None
        if reason_flags & cls.HAS_TYPE_OF_DEATH:
            death_type = reader.read_byte()
        projectile_type = None
        if reason_flags & cls.HAS_PROJECTILE_TYPE:
            projectile_type = reader.read_short()
        item_type = None
        if reason_flags & cls.HAS_ITEM_TYPE:
            item_type = reader.read_short()
        item_prefix = None
        if reason_flags & cls.HAS_ITEM_PREFIX:
            item_prefix = reader.read_byte()
        custom_reason = None
        if reason_flags & cls.HAS_CUSTOM_REASON:
            custom_reason = reader.read_dotnet_string()

        return cls(
            reason_flags,
            killer_player_id,
            killing_npc_index,
            projectile_index,
            death_type,
            projectile_type,
            item_type,
            item_prefix,
            custom_reason,
        )

    def write(self, writer: Writer) -> None:
        flags = 0
        if self.killer_player_id is not None:
            flags |= self.HAS_KILLER
        if self.killing_npc_index is not None:
            flags |= self.HAS_KILLING_NPC
        if self.projectile_index is not None:
            flags |= self.HAS_PROJECTILE_IDX
        if self.death_type is not None:
            flags |= self.HAS_TYPE_OF_DEATH
        if self.projectile_type is not None:
            flags |= self.HAS_PROJECTILE_TYPE
        if self.item_type is not None:
            flags |= self.HAS_ITEM_TYPE
        if self.item_prefix is not None:
            flags |= self.HAS_ITEM_PREFIX
        if self.custom_reason is not None:
            flags |= self.HAS_CUSTOM_REASON

        writer.write_byte(flags)

        if self.killer_player_id is not None:
            writer.write_short(self.killer_player_id)
        if self.killing_npc_index is not None:
            writer.write_short(self.killing_npc_index)
        if self.projectile_index is not None:
            writer.write_short(self.projectile_index)
        if self.death_type is not None:
            writer.write_byte(self.death_type)
        if self.projectile_type is not None:
            writer.write_short(self.projectile_type)
        if self.item_type is not None:
            writer.write_short(self.item_type)
        if self.item_prefix is not None:
            writer.write_byte(self.item_prefix)
        if self.custom_reason is not None:
            writer.write_dotnet_string(self.custom_reason)



"""
GameVersionID - autogenerated from decompiled Terraria .cs files
Terraria namespace: Terraria.ID
Terraria version: v1.4.5.5
Generator: TerrariaPyGen v1.1.1 (Maksim Stashkevich)
Generated on: 2026-02-15 09:24
"""

from enum import IntEnum, auto


class GameVersionID(IntEnum):
    WorldManifest = 299
    DisplayDollPose = 307
    DisplayDollHeldItem = 308
    OneTimeDialogue = 310
    DisplayDollMount = 311
    DisplayDollMountDye = 312
    WorldManifestV2 = 313
    HostToken = 314
    NPCHomelessDespawn = 315
    Terraria1453Release = 316
    Latest = 317
    Terraria1454Release = 317




from dataclasses import dataclass, field
from typing import cast

from terrex.item.item import Item
from terrex.net.streamer import Reader, Writer
from terrex.net.structure.vec2 import Vec2


@dataclass
class TileEntityBase:
    id: int = 0
    x: int = 0
    y: int = 0


@dataclass
class TrainingDummy(TileEntityBase):
    type: int = 0
    npc_index: int = 0


@dataclass
class ItemFrame(TileEntityBase):
    type: int = 1
    item_type: int = 0
    item_prefix: int = 0
    item_stack: int = 0


@dataclass
class LogicSensor(TileEntityBase):
    type: int = 2
    logic_check_type: int = 0
    on: bool = False


@dataclass
class DisplayDoll(TileEntityBase):
    type: int = 3
    pose: int = 0
    equip: list[Item | None] = field(default_factory=lambda: [None] * 9)
    dyes: list[Item | None] = field(default_factory=lambda: [None] * 9)
    misc: list[Item | None] = field(default_factory=lambda: [None] * 8)


@dataclass
class WeaponRack(TileEntityBase):
    type: int = 4
    item_type: int = 0
    item_prefix: int = 0
    item_stack: int = 0


@dataclass
class HatRack(TileEntityBase):
    type: int = 5
    items: list[Item | None] = field(default_factory=lambda: [None, None])
    dyes: list[Item | None] = field(default_factory=lambda: [None, None])


@dataclass
class FoodPlatter(TileEntityBase):
    type: int = 6
    item_type: int = 0
    item_prefix: int = 0
    item_stack: int = 0


@dataclass
class TeleportationPylon(TileEntityBase):
    type: int = 7


@dataclass
class DeadCellsDisplayJar(TileEntityBase):
    type: int = 8
    item_type: int = 0
    item_prefix: int = 0
    item_stack: int = 0


@dataclass
class LeashedEntityAnchorWithItem(TileEntityBase):
    item_type: int = 0


@dataclass
class KiteAnchor(LeashedEntityAnchorWithItem):
    type: int = 9
    item_type: int = 0


@dataclass
class CritterAnchor(LeashedEntityAnchorWithItem):
    type: int = 10
    item_type: int = 0


TileEntity = (
    TrainingDummy
    | ItemFrame
    | LogicSensor
    | DisplayDoll
    | WeaponRack
    | HatRack
    | FoodPlatter
    | TeleportationPylon
    | DeadCellsDisplayJar
    | KiteAnchor
    | CritterAnchor
)


def read_tile_entity(reader: Reader) -> TileEntity | None:
    typ = reader.read_byte()
    if typ < 0:
        return None
    id_ = reader.read_int()
    x = reader.read_short()
    y = reader.read_short()
    if typ == 0:
        npc_index = reader.read_ushort()
        return TrainingDummy(type=typ, id=id_, x=x, y=y, npc_index=npc_index)
    elif typ == 1:
        item_type = reader.read_ushort()
        item_prefix = reader.read_byte()
        item_stack = reader.read_ushort()
        return ItemFrame(
            type=typ,
            id=id_,
            x=x,
            y=y,
            item_type=item_type,
            item_prefix=item_prefix,
            item_stack=item_stack,
        )
    elif typ == 2:
        logic_check_type = reader.read_byte()
        on = reader.read_bool()
        return LogicSensor(type=typ, id=id_, x=x, y=y, logic_check_type=logic_check_type, on=on)
    elif typ == 3:
        bits_byte = reader.read_byte()
        bits_byte1 = reader.read_byte()
        pose = 0
        if reader.version >= 307:
            pose = reader.read_byte()
        bits_byte2 = 0
        if reader.version >= 308:
            bits_byte2 = reader.read_byte()
        has_item = False
        if reader.version == 311:
            has_item = bool(bits_byte2 & 2)
            bits_byte2 &= ~2
        num_equip = bits_byte | (256 if (bits_byte2 & 2) else 0)
        equip: list[Item | None] = cast(list[Item | None], [None] * 9)
        for i in range(9):
            if num_equip & (1 << i):
                item_id = reader.read_ushort()
                prefix = reader.read_byte()
                stack = reader.read_ushort()
                equip[i] = Item(
                    item_id=item_id,
                    prefix=prefix,
                    stacks=stack,
                    net_id=0,
                    position=Vec2(0, 0),
                    velocity=Vec2(0, 0),
                )
        num_dyes = bits_byte1 | (256 if (bits_byte2 & 4) else 0)
        dyes: list[Item | None] = cast(list[Item | None], [None] * 9)
        for j in range(9):
            if num_dyes & (1 << j):
                item_id = reader.read_ushort()
                prefix = reader.read_byte()
                stack = reader.read_ushort()
                dyes[j] = Item(
                    item_id=item_id,
                    prefix=prefix,
                    stacks=stack,
                    net_id=0,
                    position=Vec2(0, 0),
                    velocity=Vec2(0, 0),
                )
        misc: list[Item | None] = cast(list[Item | None], [None] * 8)
        for k in range(8):
            if bits_byte2 & (1 << k):
                item_id = reader.read_ushort()
                prefix = reader.read_byte()
                stack = reader.read_ushort()
                misc[k] = Item(
                    item_id=item_id,
                    prefix=prefix,
                    stacks=stack,
                    net_id=0,
                    position=Vec2(0, 0),
                    velocity=Vec2(0, 0),
                )
        if has_item:
            item_id = reader.read_ushort()
            prefix = reader.read_byte()
            stack = reader.read_ushort()
            equip[8] = Item(
                item_id=item_id,
                prefix=prefix,
                stacks=stack,
                net_id=0,
                position=Vec2(0, 0),
                velocity=Vec2(0, 0),
            )
        return DisplayDoll(type=typ, id=id_, x=x, y=y, pose=pose, equip=equip, dyes=dyes, misc=misc)
    elif typ == 4:
        item_type = reader.read_ushort()
        item_prefix = reader.read_byte()
        item_stack = reader.read_ushort()
        return WeaponRack(
            type=typ,
            id=id_,
            x=x,
            y=y,
            item_type=item_type,
            item_prefix=item_prefix,
            item_stack=item_stack,
        )
    elif typ == 5:
        flags = reader.read_byte()
        items: list[Item | None] = cast(list[Item | None], [None] * 2)
        hatrack_dyes: list[Item | None] = cast(list[Item | None], [None] * 2)
        for i in range(2):
            if flags & (1 << i):
                item_id = reader.read_ushort()
                prefix = reader.read_byte()
                stack = reader.read_ushort()
                items[i] = Item(
                    item_id=item_id,
                    prefix=prefix,
                    stacks=stack,
                    net_id=0,
                    position=Vec2(0, 0),
                    velocity=Vec2(0, 0),
                )
        for j in range(2):
            if flags & (1 << (j + 2)):
                item_id = reader.read_ushort()
                prefix = reader.read_byte()
                stack = reader.read_ushort()
                hatrack_dyes[j] = Item(
                    item_id=item_id,
                    prefix=prefix,
                    stacks=stack,
                    net_id=0,
                    position=Vec2(0, 0),
                    velocity=Vec2(0, 0),
                )
        return HatRack(type=typ, id=id_, x=x, y=y, items=items, dyes=hatrack_dyes)
    elif typ == 6:
        item_type = reader.read_ushort()
        item_prefix = reader.read_byte()
        item_stack = reader.read_ushort()
        return FoodPlatter(
            type=typ,
            id=id_,
            x=x,
            y=y,
            item_type=item_type,
            item_prefix=item_prefix,
            item_stack=item_stack,
        )
    elif typ == 7:
        return TeleportationPylon(type=typ, id=id_, x=x, y=y)
    # added on v1.4.5.0+
    elif typ == 8:
        item_type = reader.read_ushort()
        item_prefix = reader.read_byte()
        item_stack = reader.read_ushort()
        return DeadCellsDisplayJar(
            type=typ,
            id=id_,
            x=x,
            y=y,
            item_type=item_type,
            item_prefix=item_prefix,
            item_stack=item_stack,
        )
    elif typ == 9:
        item_type = reader.read_ushort()
        return KiteAnchor(type=typ, id=id_, x=x, y=y, item_type=item_type)
    elif typ == 10:
        item_type = reader.read_ushort()
        return CritterAnchor(type=typ, id=id_, x=x, y=y, item_type=item_type)
    else:
        print(f"Unknown TileEntity type: {typ}")
        return None


def write_tile_entity(entity: TileEntity, writer: Writer) -> None:
    # TODO: implement based on type
    pass



from terrex.net.tile_stack import TileStack


class World:
    time: int = 0
    day_info: int = 0
    moon_phase: int = 0
    max_tiles_x: int = 0
    max_tiles_y: int = 0
    spawn_x: int = 0
    spawn_y: int = 0
    world_surface: int = 0
    rock_layer: int = 0
    world_id: int = 0
    world_name: str = ""
    game_mode: int = 0
    world_unique_id: list[int] = []
    world_generator_version: list[int] = []
    moon_type: int = 0
    tree_background: int = 0
    corruption_background: int = 0
    jungle_background: int = 0
    snow_background: int = 0
    hallow_background: int = 0
    crimson_background: int = 0
    desert_background: int = 0
    ocean_background: int = 0
    unknown_background: list[int] = []
    ice_back_style: int = 0
    jungle_back_style: int = 0
    hell_back_style: int = 0
    wind_speed_set: float = 0.0
    cloud_number: int = 0
    trees: list[int] = []
    tree_styles: list[int] = []
    cave_backs: list[int] = []
    cave_back_styles: list[int] = []
    forest_tree_top_styles: list[int] = []
    corruption_tree_top_style: int = 0
    jungle_tree_top_style: int = 0
    snow_tree_top_style: int = 0
    hallow_tree_top_style: int = 0
    crimson_tree_top_style: int = 0
    desert_tree_top_style: int = 0
    ocean_tree_top_style: int = 0
    glowing_mushroom_tree_top_style: int = 0
    underworld_tree_top_style: int = 0
    rain: float = 0.0
    event_info: int = 0
    ore_tiers_tiles: list[int] = []
    invasion_type: int = 0
    lobby_id: int = 0
    sandstorm_severity: float = 0.0

    # todo: move this
    items: object = {}
    item_owner_index: object = {}

    tiles: TileStack
    chests: dict = {}
    signs: dict = {}
    tile_entities: list = []

    # always false because we connected to generated server world
    generating_world: bool = False

    # players info
    players: dict = {}
    my_player_id: int = -1

    # flags for special world seeds
    # https://terraria.wiki.gg/wiki/Special_world_seeds
    drunk_world: bool = False
    get_good_world: bool = False
    tenth_anniversary_world: bool = False
    dont_starve_world: bool = False
    not_the_bees_world: bool = False
    remix_world: bool = False
    no_traps_world: bool = False
    zenith_world: bool = False
    skyblock_world: bool = False
    vampire_seed: bool = False
    infected_seed: bool = False
    team_based_spawns_seed: bool = False
    dual_dungeons_seed: bool = False

    def __init__(self):
        from terrex.entity.tile_entity.tile_entity import TileEntity
        from terrex.net.structure.chest import Chest
        from terrex.net.structure.sign import Sign
        from terrex.net.tile_stack import TileStack
        from terrex.player.player import Player

        self.tiles: TileStack = TileStack()
        self.chests: dict[int, Chest] = {}
        self.signs: dict[int, Sign] = {}
        self.tile_entities: list[TileEntity] = []

        self.players: dict[int, Player] = {}

    @classmethod
    def underworld_layer(cls) -> int:
        return cls.max_tiles_y - 200



"""
TeleportationSide - autogenerated from decompiled Terraria .cs files
Terraria namespace: Terraria.ID
Terraria version: v1.4.5.5
Generator: TerrariaPyGen v1.1.1 (Maksim Stashkevich)
Generated on: 2026-02-15 09:24
"""

from enum import IntEnum, auto


class TeleportationSide(IntEnum):
    Entry = 0
    Exit = 1




"""
WallID - autogenerated from decompiled Terraria .cs files
Terraria namespace: Terraria.ID
Terraria version: v1.4.5.5
Generator: TerrariaPyGen v1.1.1 (Maksim Stashkevich)
Generated on: 2026-02-15 09:24
"""

from terrex.id.set_factory import SetFactory
from enum import IntEnum, auto


class WallID:
    Count = 367

    class Conversion(IntEnum):
        NoneValue = 0
        Stone = 1
        DirtUnsafe = 2
        EbonstoneUnsafe = 3
        Wood = 4
        GrayBrick = 5
        RedBrick = 6
        BlueDungeonUnsafe = 7
        GreenDungeonUnsafe = 8
        PinkDungeonUnsafe = 9
        GoldBrick = 10
        SilverBrick = 11
        CopperBrick = 12
        HellstoneBrickUnsafe = 13
        ObsidianBrickUnsafe = 14
        MudUnsafe = 15
        Dirt = 16
        BlueDungeon = 17
        GreenDungeon = 18
        PinkDungeon = 19
        ObsidianBrick = 20
        Glass = 21
        PearlstoneBrick = 22
        IridescentBrick = 23
        MudstoneBrick = 24
        CobaltBrick = 25
        MythrilBrick = 26
        Planked = 27
        PearlstoneBrickUnsafe = 28
        CandyCane = 29
        GreenCandyCane = 30
        SnowBrick = 31
        AdamantiteBeam = 32
        DemoniteBrick = 33
        SandstoneBrick = 34
        EbonstoneBrick = 35
        RedStucco = 36
        YellowStucco = 37
        GreenStucco = 38
        Gray = 39
        SnowWallUnsafe = 40
        Ebonwood = 41
        RichMaogany = 42
        Pearlwood = 43
        RainbowBrick = 44
        TinBrick = 45
        TungstenBrick = 46
        PlatinumBrick = 47
        AmethystUnsafe = 48
        TopazUnsafe = 49
        SapphireUnsafe = 50
        EmeraldUnsafe = 51
        RubyUnsafe = 52
        DiamondUnsafe = 53
        CaveUnsafe = 54
        Cave2Unsafe = 55
        Cave3Unsafe = 56
        Cave4Unsafe = 57
        Cave5Unsafe = 58
        Cave6Unsafe = 59
        LivingLeaf = 60
        Cave7Unsafe = 61
        SpiderUnsafe = 62
        GrassUnsafe = 63
        JungleUnsafe = 64
        FlowerUnsafe = 65
        Grass = 66
        Jungle = 67
        Flower = 68
        CorruptGrassUnsafe = 69
        HallowedGrassUnsafe = 70
        IceUnsafe = 71
        Cactus = 72
        Cloud = 73
        Mushroom = 74
        Bone = 75
        Slime = 76
        Flesh = 77
        LivingWood = 78
        ObsidianBackUnsafe = 79
        MushroomUnsafe = 80
        CrimsonGrassUnsafe = 81
        DiscWall = 82
        CrimstoneUnsafe = 83
        IceBrick = 84
        Shadewood = 85
        HiveUnsafe = 86
        LihzahrdBrickUnsafe = 87
        PurpleStainedGlass = 88
        YellowStainedGlass = 89
        BlueStainedGlass = 90
        GreenStainedGlass = 91
        RedStainedGlass = 92
        RainbowStainedGlass = 93
        BlueDungeonSlabUnsafe = 94
        BlueDungeonTileUnsafe = 95
        PinkDungeonSlabUnsafe = 96
        PinkDungeonTileUnsafe = 97
        GreenDungeonSlabUnsafe = 98
        GreenDungeonTileUnsafe = 99
        BlueDungeonSlab = 100
        BlueDungeonTile = 101
        PinkDungeonSlab = 102
        PinkDungeonTile = 103
        GreenDungeonSlab = 104
        GreenDungeonTile = 105
        WoodenFence = 106
        MetalFence = 107
        Hive = 108
        PalladiumColumn = 109
        BubblegumBlock = 110
        TitanstoneBlock = 111
        LihzahrdBrick = 112
        Pumpkin = 113
        Hay = 114
        SpookyWood = 115
        ChristmasTreeWallpaper = 116
        OrnamentWallpaper = 117
        CandyCaneWallpaper = 118
        FestiveWallpaper = 119
        StarsWallpaper = 120
        SquigglesWallpaper = 121
        SnowflakeWallpaper = 122
        KrampusHornWallpaper = 123
        BluegreenWallpaper = 124
        GrinchFingerWallpaper = 125
        FancyGrayWallpaper = 126
        IceFloeWallpaper = 127
        MusicWallpaper = 128
        PurpleRainWallpaper = 129
        RainbowWallpaper = 130
        SparkleStoneWallpaper = 131
        StarlitHeavenWallpaper = 132
        BubbleWallpaper = 133
        CopperPipeWallpaper = 134
        DuckyWallpaper = 135
        Waterfall = 136
        Lavafall = 137
        EbonwoodFence = 138
        RichMahoganyFence = 139
        PearlwoodFence = 140
        ShadewoodFence = 141
        WhiteDynasty = 142
        BlueDynasty = 143
        ArcaneRunes = 144
        IronFence = 145
        CopperPlating = 146
        StoneSlab = 147
        Sail = 148
        BorealWood = 149
        BorealWoodFence = 150
        PalmWood = 151
        PalmWoodFence = 152
        AmberGemspark = 153
        AmethystGemspark = 154
        DiamondGemspark = 155
        EmeraldGemspark = 156
        AmberGemsparkOff = 157
        AmethystGemsparkOff = 158
        DiamondGemsparkOff = 159
        EmeraldGemsparkOff = 160
        RubyGemsparkOff = 161
        SapphireGemsparkOff = 162
        TopazGemsparkOff = 163
        RubyGemspark = 164
        SapphireGemspark = 165
        TopazGemspark = 166
        TinPlating = 167
        Confetti = 168
        ConfettiBlack = 169
        CaveWall = 170
        CaveWall2 = 171
        Honeyfall = 172
        ChlorophyteBrick = 173
        CrimtaneBrick = 174
        ShroomitePlating = 175
        MartianConduit = 176
        HellstoneBrick = 177
        MarbleUnsafe = 178
        MarbleBlock = 179
        GraniteUnsafe = 180
        GraniteBlock = 181
        MeteoriteBrick = 182
        Marble = 183
        Granite = 184
        Cave8Unsafe = 185
        Crystal = 186
        Sandstone = 187
        CorruptionUnsafe1 = 188
        CorruptionUnsafe2 = 189
        CorruptionUnsafe3 = 190
        CorruptionUnsafe4 = 191
        CrimsonUnsafe1 = 192
        CrimsonUnsafe2 = 193
        CrimsonUnsafe3 = 194
        CrimsonUnsafe4 = 195
        DirtUnsafe1 = 196
        DirtUnsafe2 = 197
        DirtUnsafe3 = 198
        DirtUnsafe4 = 199
        HallowUnsafe1 = 200
        HallowUnsafe2 = 201
        HallowUnsafe3 = 202
        HallowUnsafe4 = 203
        JungleUnsafe1 = 204
        JungleUnsafe2 = 205
        JungleUnsafe3 = 206
        JungleUnsafe4 = 207
        LavaUnsafe1 = 208
        LavaUnsafe2 = 209
        LavaUnsafe3 = 210
        LavaUnsafe4 = 211
        RocksUnsafe1 = 212
        RocksUnsafe2 = 213
        RocksUnsafe3 = 214
        RocksUnsafe4 = 215
        HardenedSand = 216
        CorruptHardenedSand = 217
        CrimsonHardenedSand = 218
        HallowHardenedSand = 219
        CorruptSandstone = 220
        CrimsonSandstone = 221
        HallowSandstone = 222
        DesertFossil = 223
        LunarBrickWall = 224
        CogWall = 225
        SandFall = 226
        SnowFall = 227
        SillyBalloonPinkWall = 228
        SillyBalloonPurpleWall = 229
        SillyBalloonGreenWall = 230
        IronBrick = 231
        LeadBrick = 232
        LesionBlock = 233
        CrimstoneBrick = 234
        SmoothSandstone = 235
        Spider = 236
        SolarBrick = 237
        VortexBrick = 238
        NebulaBrick = 239
        StardustBrick = 240
        OrangeStainedGlass = 241
        GoldStarryGlassWall = 242
        BlueStarryGlassWall = 243
        LivingWoodUnsafe = 244
        WroughtIronFence = 245
        EbonstoneEcho = 246
        MudWallEcho = 247
        PearlstoneEcho = 248
        SnowWallEcho = 249
        AmethystEcho = 250
        TopazEcho = 251
        SapphireEcho = 252
        EmeraldEcho = 253
        RubyEcho = 254
        DiamondEcho = 255
        Cave1Echo = 256
        Cave2Echo = 257
        Cave3Echo = 258
        Cave4Echo = 259
        Cave5Echo = 260
        Cave6Echo = 261
        Cave7Echo = 262
        SpiderEcho = 263
        CorruptGrassEcho = 264
        HallowedGrassEcho = 265
        IceEcho = 266
        ObsidianBackEcho = 267
        CrimsonGrassEcho = 268
        CrimstoneEcho = 269
        CaveWall1Echo = 270
        CaveWall2Echo = 271
        MarbleEchoUnused = 272
        GraniteEchoUnused = 273
        Cave8Echo = 274
        SandstoneEcho = 275
        Corruption1Echo = 276
        Corruption2Echo = 277
        Corruption3Echo = 278
        Corruption4Echo = 279
        Crimson1Echo = 280
        Crimson2Echo = 281
        Crimson3Echo = 282
        Crimson4Echo = 283
        Dirt1Echo = 284
        Dirt2Echo = 285
        Dirt3Echo = 286
        Dirt4Echo = 287
        Hallow1Echo = 288
        Hallow2Echo = 289
        Hallow3Echo = 290
        Hallow4Echo = 291
        Jungle1Echo = 292
        Jungle2Echo = 293
        Jungle3Echo = 294
        Jungle4Echo = 295
        Lava1Echo = 296
        Lava2Echo = 297
        Lava3Echo = 298
        Lava4Echo = 299
        Rocks1Echo = 300
        Rocks2Echo = 301
        Rocks3Echo = 302
        Rocks4Echo = 303
        HardenedSandEcho = 304
        CorruptHardenedSandEcho = 305
        CrimsonHardenedSandEcho = 306
        HallowHardenedSandEcho = 307
        CorruptSandstoneEcho = 308
        CrimsonSandstoneEcho = 309
        HallowSandstoneEcho = 310
        DesertFossilEcho = 311
        BambooBlockWall = 312
        LargeBambooBlockWall = 313
        AmberStoneWallEcho = 314
        BambooFence = 315
        AshWood = 316
        AshWoodFence = 317
        EchoWall = 318
        ReefWall = 319
        PoopWall = 320
        ShimmerBlockWall = 321
        ShimmerBrickWall = 322
        LunarRustBrickWall = 323
        DarkCelestialBrickWall = 324
        AstraBrickWall = 325
        CosmicEmberBrickWall = 326
        CryocoreBrickWall = 327
        MercuryBrickWall = 328
        StarRoyaleBrickWall = 329
        HeavenforgeBrickWall = 330
        AncientBlueBrickWall = 331
        AncientGreenBrickWall = 332
        AncientPinkBrickWall = 333
        AncientGoldBrickWall = 334
        AncientSilverBrickWall = 335
        AncientCopperBrickWall = 336
        AncientObsidianBrickWall = 337
        AncientHellstoneBrickWall = 338
        AncientCobaltBrickWall = 339
        AncientMythrilBrickWall = 340
        LavaMossBlockWall = 341
        ArgonMossBlockWall = 342
        KryptonMossBlockWall = 343
        XenonMossBlockWall = 344
        VioletMossBlockWall = 345
        RainbowMossBlockWall = 346
        Shimmerfall = 347
        FallenStarWall = 348
        StoneUnsafe = 349
        UnbreakableBlockWall = 350
        FeywoodWall = 351
        HallowedBrickWall = 352
        PineTreeBlockWall = 353
        EasterBlockWall = 354
        GothicBrickWall = 355
        FlinxFurBlockWall = 356
        JellyfishBlockWall = 357
        PineWoodBlockWall = 358
        HarpyBlockWall = 359
        MoonplateBlockWall = 360
        LibrarianBlockWall = 361
        SpikeBlockWall = 362
        OfficeBlockWall = 363
        ForbiddenBlockWall = 364
        WaterBlockWall = 365
        BoulderBlockWall = 366
        Count = 367


class WallIDSets:
    class Conversion:
        factory = SetFactory(WallID.Count)
        Grass = factory.create_bool_set(False, 63, 64, 65, 66, 67, 68, 69, 70, 81, 264, 265, 268)
        Stone = factory.create_bool_set(False, 1, 61, 185, 3, 28, 83, 262, 274, 246, 248, 269, 349)
        Dirt = factory.create_bool_set(False, 2, 16)
        Snow = factory.create_bool_set(False, 40, 249)
        Ice = factory.create_bool_set(False, 71, 266)
        Sandstone = factory.create_bool_set(False, 187, 220, 222, 221, 275, 308, 310, 309)
        HardenedSand = factory.create_bool_set(False, 216, 217, 219, 218, 304, 305, 307, 306)
        PureSand = factory.create_bool_set(False, 216, 187, 304, 275)
        NewWall1 = factory.create_bool_set(False, 188, 192, 200, 204, 212, 276, 280, 288, 292, 300)
        NewWall2 = factory.create_bool_set(False, 189, 193, 201, 205, 213, 277, 281, 289, 293, 301)
        NewWall3 = factory.create_bool_set(False, 190, 194, 202, 206, 214, 278, 282, 290, 294, 302)
        NewWall4 = factory.create_bool_set(False, 191, 195, 203, 207, 215, 279, 283, 291, 295, 303)

    class Main:
        factory = SetFactory(WallID.Count)
        DualDungeonsJungleBiomeWalls = factory.create_bool_set(False, 64, 204, 205, 206, 207, 15)
        CanBeConvertedToGlowingMushroom = factory.create_bool_set(False, 64, 67, 15, 247)
        AllowsUndergroundDesertEnemiesToSpawn = factory.create_bool_set(False, 187, 220, 222, 221, 216, 217, 219, 218, 223)
        Transparent = factory.create_bool_set(False, 0, 88, 89, 90, 91, 92, 241, 93)
        Glass = factory.create_bool_set(False, 21, 88, 89, 90, 91, 92, 241, 93, 243, 242)
        SpreadsCorruption = factory.create_bool_set(False, 69, 217, 220, 3)
        SpreadsCrimson = factory.create_bool_set(False, 83, 81, 218, 221)
        SpreadsHallow = factory.create_bool_set(False, 70, 219, 222, 28)
        AllowsWind = factory.create_bool_set(False, 0, 150, 138, 145, 107, 152, 140, 139, 141, 106, 245, 315, 317)
        Fences = factory.create_bool_set(False, 150, 138, 145, 107, 152, 140, 139, 141, 106, 245, 315, 317)
        AllowsPlantsToGrow = factory.create_bool_set(False, 0, 150, 138, 145, 107, 152, 140, 139, 141, 106, 245, 315, 317, 63, 64, 65, 66, 67, 68, 69, 81, 70, 264, 268, 265, 74, 80)
        CannotBeReplacedByWallSpread = factory.create_bool_set(False, 4, 40, 3, 83, 87, 244, 34)
        WallSpreadStopsAtAir = factory.create_bool_set(False, 63, 62)
        BlendType = factory.create_int_set(-1, 66, 63, 68, 63, 65, 63, 16, 2, 59, 2, 261, 2, 284, 196, 285, 197, 286, 198, 287, 199, 256, 54, 257, 55, 258, 56, 259, 57, 260, 58, 262, 61, 274, 185, 300, 212, 301, 213, 302, 214, 303, 215, 296, 208, 297, 209, 298, 210, 299, 211, 48, 1, 49, 1, 50, 1, 51, 1, 52, 1, 53, 1, 250, 1, 251, 1, 252, 1, 253, 1, 254, 1, 255, 1, 69, 264, 3, 246, 217, 305, 220, 308, 188, 276, 189, 277, 190, 278, 191, 279, 81, 77, 268, 77, 83, 269, 218, 306, 221, 309, 192, 280, 193, 281, 194, 282, 195, 283, 70, 265, 28, 248, 219, 307, 222, 310, 200, 288, 201, 289, 202, 290, 203, 291, 15, 247, 64, 67, 204, 292, 205, 293, 206, 294, 207, 295, 86, 108, 87, 112, 40, 249, 71, 266, 216, 304, 187, 275, 62, 263, 80, 74, 180, 184, 178, 183, 79, 267, 20, 14, 7, 17, 94, 17, 95, 17, 8, 18, 98, 18, 99, 18, 9, 19, 96, 19, 97, 19, 349, 1, 244, 78)



"""
GoreID - autogenerated from decompiled Terraria .cs files
Terraria namespace: Terraria.ID
Terraria version: v1.4.5.5
Generator: TerrariaPyGen v1.1.1 (Maksim Stashkevich)
Generated on: 2026-02-15 09:24
"""

from terrex.id.set_factory import SetFactory
from enum import IntEnum, auto


class GoreID(IntEnum):
    Cloud1 = 11
    Cloud2 = 12
    Cloud3 = 13
    FireImpHead = 45
    FireImpArm = 46
    FireImpLeg = 47
    PotLightBrown1 = 51
    PotLightBrown2 = 52
    PotLightBrown3 = 53
    Smoke1 = 61
    Smoke2 = 62
    Smoke3 = 63
    HornetHead = 70
    HornetBody = 71
    PotDarkBrown1 = 166
    PotDarkBrown2 = 167
    PotDarkBrown3 = 168
    PotOrnate1 = 169
    PotOrnate2 = 170
    PotOrnate3 = 171
    PotOrnateBrown1 = 172
    PotOrnateBrown2 = 173
    PotOrnateBrown3 = 174
    PotIcy1 = 197
    PotIcy2 = 198
    PotJungle1 = 199
    PotJungle2 = 200
    PotSkull1 = 201
    PotSkull2 = 202
    PotObsidian1 = 203
    PotObsidian2 = 204
    PotPyramid1 = 217
    PotPyramid2 = 218
    PotPyramid3 = 219
    MossHornetHead = 229
    MossHornetBody = 230
    PotTemple1 = 315
    PotTemple2 = 316
    FartCloud1 = 435
    FartCloud2 = 436
    FartCloud3 = 437
    ChargedBlasterRing = 618
    MoonLordHeart1 = 619
    MoonLordHeart2 = 620
    MoonLordHeart3 = 621
    MoonLordHeart4 = 622
    GoblinArcherHead = 623
    GoblinArcherHand = 624
    GoblinArcherLeg = 625
    GoblinPeonHead = 626
    GoblinPeonHand = 627
    GoblinPeonLeg = 628
    GoblinScoutHead = 629
    GoblinScoutHand = 630
    GoblinScoutLeg = 631
    GoblinThiefHead = 632
    GoblinThiefHand = 633
    GoblinThiefLeg = 634
    GoblinWarriorHead = 635
    GoblinWarriorHand = 636
    GoblinWarriorLeg = 637
    SkeletonMerchantHead = 638
    SkeletonMerchantBag = 639
    SkeletonMerchantChunk1 = 640
    SkeletonMerchantChunk2 = 641
    SkeletonMerchantChunk3 = 642
    ButcherHead = 643
    ButcherSaw = 644
    ButcherArm = 645
    ButcherLeg = 646
    CreatureFromTheDeepHead = 647
    CreatureFromTheDeepArm = 648
    CreatureFromTheDeepLeg = 649
    FritzHead = 650
    FritzArm = 651
    FritzLeg = 652
    NailheadHead = 653
    NailheadArm = 654
    NailheadLeg = 655
    PsychoHead = 656
    PsychoArm = 657
    PsychoLeg = 658
    PsychoKnife = 659
    DeadlySphere1 = 660
    DeadlySphere2 = 661
    DrManFlyHead = 662
    DrManFlyArm = 663
    DrManFlyLeg = 664
    ThePossessedHead = 665
    ThePossessedArm = 666
    ThePossessedLeg = 667
    CrimsonBunnyHead = 668
    CrimsonBunnyLeg = 669
    CrimsonGoldfishHead = 670
    CrimsonGoldfishTail = 671
    CrimsonPenguinBody = 672
    CrimsonPenguinHead = 673
    CrimsonPenguinLeg = 674
    GoblinSummonerHead = 675
    GoblinSummonerArm = 676
    GoblinSummonerItem = 677
    GoblinSummonerLeg = 678
    ShadowflameApparation = 679
    ShadowflameApparation2 = 680
    MothronSpawn1 = 681
    MothronSpawn2 = 682
    MothronSpawn3 = 683
    MothronEgg1 = 684
    MothronEgg2 = 685
    MothronEgg3 = 686
    Mothron1 = 687
    Mothron2 = 688
    Mothron3 = 689
    Mothron4 = 690
    Mothron5 = 691
    MedusaHead = 692
    MedusaArm = 693
    MedusaLeg = 694
    GreekSkeletonHead = 695
    GreekSkeletonArm = 696
    GreekSkeletonLeg = 697
    GreekPot1 = 698
    GreekPot2 = 699
    GreekPot3 = 700
    GreekPot4 = 701
    GreekPot5 = 702
    GreekPot6 = 703
    ToxicFlask = 704
    ToxicFlask2 = 705
    WaterDrip = 706
    WaterDripCorrupt = 707
    WaterDripJungle = 708
    WaterDripHallow = 709
    WaterDripIce = 710
    WaterDripDesert = 711
    WaterDripUnderground = 712
    WaterDripCavern = 713
    WaterDripBlood = 714
    WaterDripCrimson = 715
    LavaDrip = 716
    HoneyDrip = 717
    GraniteGolemHead = 718
    GraniteGolemBody = 719
    GraniteGolemArm = 720
    GraniteGolemLeg = 721
    BloodZombieHead = 722
    BloodZombieChunk = 723
    BloodZombieChunk2 = 724
    DripplerChunk = 725
    DripplerChunk2 = 726
    DripplerChunk3 = 727
    LunarTablet1 = 728
    LunarTablet2 = 729
    LunarTablet3 = 730
    LunarTablet4 = 731
    LunarTablet5 = 732
    LunarTablet6 = 733
    KingSlimeCrown = 734
    CrawDadClaw = 735
    CrawDadEye = 736
    CrawDad2Claw = 737
    CrawDad2Eye = 738
    GiantShelly = 739
    GiantShelly2 = 740
    SalamanderArm = 741
    SalamanderHead = 750
    SalamanderLeg = 759
    TaxCollectorHead = 768
    TaxCollectorArm = 769
    TaxCollectorCane = 770
    TaxCollectorLeg = 771
    StardustSoldier1 = 772
    StardustSoldier2 = 773
    StardustSoldier3 = 774
    StardustSpider1 = 775
    StardustSpider2 = 776
    StardustSpider3 = 777
    StardustJellyfish1 = 778
    StardustJellyfish2 = 779
    StardustJellyfish3 = 780
    StardustJellyfish4 = 781
    NebulaBrain1 = 782
    NebulaBrain2 = 783
    NebulaBrain3 = 784
    NebulaHeadcrab1 = 785
    NebulaHeadcrab2 = 786
    NebulaHeadcrab3 = 787
    NebulaEye1 = 788
    NebulaEye2 = 789
    NebulaBeast1 = 790
    NebulaBeast2 = 791
    NebulaBeast3 = 792
    NebulaSoldier1 = 793
    NebulaSoldier2 = 794
    NebulaSoldier3 = 795
    VortexRifleman1 = 796
    VortexRifleman2 = 797
    VortexRifleman3 = 798
    VortexHornetQueen1 = 799
    VortexHornetQueen2 = 800
    VortexHornetQueen3 = 801
    VortexHornet1 = 802
    VortexHornet2 = 803
    VortexHornet3 = 804
    VortexHornet4 = 805
    VortexLarva1 = 806
    VortexLarve2 = 807
    VortexSoldier1 = 808
    VortexSoldier2 = 809
    VortexSoldier3 = 810
    GiantWalkingAntlion1 = 811
    GiantWalkingAntlion2 = 812
    GiantWalkingAntlion3 = 813
    GiantWalkingAntlion4 = 814
    GiantFlyingAntlion1 = 815
    GiantFlyingAntlion2 = 816
    GiantFlyingAntlion3 = 817
    GiantFlyingAntlion4 = 818
    DuneSplicerHead = 819
    DuneSplicerBody = 820
    DuneSplicerTail = 821
    TombCrawlerHead = 822
    TombCrawlerBody = 823
    TombCrawlerTail = 824
    ChimneySmoke1 = 825
    ChimneySmoke2 = 826
    ChimneySmoke3 = 827
    SolarWormHead = 828
    SolarWormBody = 829
    SolarWormTail = 830
    SolarBeast1 = 831
    SolarBeast2 = 832
    SolarBeast3 = 833
    SolarBeast4 = 834
    SolarSpearman1 = 835
    SolarSpearman2 = 836
    SolarRoller1 = 837
    SolarRoller2 = 838
    SolarRoller3 = 839
    SolarRoller4 = 840
    SolarMeteor1 = 841
    SolarMeteor2 = 842
    SolarMeteor3 = 843
    SolarSoldier1 = 844
    SolarSoldier2 = 845
    SolarSoldier3 = 846
    SolarSoldier4 = 847
    MartianWalker1 = 848
    MartianWalker2 = 849
    MartianWalker3 = 850
    MartianWalker4 = 851
    MartianWalker5 = 852
    PirateShip1 = 853
    PirateShip2 = 854
    PirateShip3 = 855
    PirateShip4 = 856
    PirateShip5 = 857
    PirateShip6 = 858
    PirateShip7 = 859
    PirateShip8 = 860
    PirateShip9 = 861
    PirateShip10 = 862
    DesertGhoul1 = 863
    DesertGhoul2 = 864
    DesertGhoul3 = 865
    DesertGhoul4 = 866
    DesertGhoulCorruption1 = 867
    DesertGhoulCorruption2 = 868
    DesertGhoulCorruption3 = 869
    DesertGhoulCorruption4 = 870
    DesertGhoulCrimson1 = 871
    DesertGhoulCrimson2 = 872
    DesertGhoulCrimson3 = 873
    DesertGhoulCrimson4 = 874
    DesertLamia1 = 875
    DesertLamia2 = 876
    DesertLamia3 = 877
    DesertLamia4 = 878
    DesertGhoulHallow1 = 879
    DesertGhoulHallow2 = 880
    DesertGhoulHallow3 = 881
    DesertGhoulHallow4 = 882
    DesertScorpion1 = 883
    DesertScorpion2 = 884
    DesertScorpion3 = 885
    DesertScorpion4 = 886
    DesertScorpion5 = 887
    DesertBeast1 = 888
    DesertBeast2 = 889
    DesertBeast3 = 890
    DesertBeast4 = 891
    PirateShip11 = 892
    PirateShip12 = 893
    DemonTaxCollectorHead = 894
    DemonTaxCollectorArm = 895
    DemonTaxCollectorLeg = 896
    SquirrelRed = 897
    MartianProbe1 = 898
    MartianProbe2 = 899
    Cultist1 = 900
    Cultist2 = 901
    CultistBoss1 = 902
    CultistBoss2 = 903
    GigaZapperHead = 904
    GigaZapperHand = 905
    GigaZapperLeg = 906
    SillyBallonPink = 907
    SillyBalloonPurple = 908
    SillyBalloonGreen = 909
    TreeLeaf_Normal = 910
    TreeLeaf_Palm = 911
    TreeLeaf_Mushroom = 912
    TreeLeaf_Boreal = 913
    TreeLeaf_Jungle = 914
    TreeLeaf_Corruption = 915
    TreeLeaf_Crimson = 916
    TreeLeaf_Hallow = 917
    TreeLeaf_HallowLast = 925
    PartyHatBlue = 926
    PartyHatDemolitionist = 927
    PartyHatDyeTrader = 928
    PartyHatMerchant = 929
    PartyHatNurse = 930
    PartyHatPainter = 931
    PartyHatPirate = 932
    PartyHatSanta = 933
    PartyHatSkeletonMerchant = 934
    PartyHatSteampunker = 935
    PartyHatStylist = 936
    PartyHatTravellingMerchant = 937
    PartyHatWizard = 938
    PartyHatPink = 939
    PartyHatCyan = 940
    PartyHatPurple = 941
    PartyHatWhite = 942
    SandDrip = 943
    Sandshark1 = 944
    Sandshark2 = 945
    Sandshark3 = 946
    Sandshark4 = 947
    Sandshark1Corrupt = 948
    Sandshark2Corrupt = 949
    Sandshark3Corrupt = 950
    Sandshark4Corrupt = 951
    Sandshark1Crimson = 952
    Sandshark2Crimson = 953
    Sandshark3Crimson = 954
    Sandshark4Crimson = 955
    Sandshark1Hallowed = 956
    Sandshark2Hallowed = 957
    Sandshark3Hallowed = 958
    Sandshark4Hallowed = 959
    SandElemental1 = 960
    SandElemental2 = 961
    SandElemental3 = 962
    SandElemental4 = 963
    Tumbleweed1 = 964
    Tumbleweed2 = 965
    Tumbleweed3 = 966
    Bartender1 = 967
    Bartender2 = 968
    Bartender3 = 969
    DD2Skeleton1 = 970
    DD2Skeleton2 = 971
    DD2Skeleton3 = 972
    DD2Skeleton4 = 973
    DD2WyvernT1_1 = 974
    DD2WyvernT1_2 = 975
    DD2WyvernT1_3 = 976
    DD2WyvernT1_4 = 977
    DD2WyvernT1_5 = 978
    DD2DrakinT3_1 = 979
    DD2DrakinT3_2 = 980
    DD2DrakinT3_3 = 981
    DD2DrakinT3_4 = 982
    DD2GoblinT1_1 = 983
    DD2GoblinT1_2 = 984
    DD2GoblinT1_3 = 985
    DD2GoblinT1_4 = 986
    DD2JavelinThrowerT1_1 = 987
    DD2JavelinThrowerT1_2 = 988
    DD2JavelinThrowerT1_3 = 989
    DD2JavelinThrowerT1_4 = 990
    DD2JavelinThrowerT1_5 = 991
    DD2JavelinThrowerT1_6 = 992
    DD2Kobold1 = 993
    DD2Kobold2 = 994
    DD2Kobold3 = 995
    DD2Kobold4 = 996
    LightningBugT3_1 = 997
    LightningBugT3_2 = 998
    LightningBugT3_3 = 999
    DD2Ogre1 = 1000
    DD2Ogre2 = 1001
    DD2Ogre3 = 1002
    DD2Ogre4 = 1003
    DD2Ogre5 = 1004
    DD2Ogre6 = 1005
    DD2Ogre6Alt = 1006
    Pages = 1007
    PageScrap = 1008
    DD2DarkMageT1_1 = 1009
    DD2DarkMageT1_2 = 1010
    DD2DarkMageT1_3 = 1011
    DD2DarkMageT1_4 = 1012
    DD2DarkMageT1_5 = 1013
    DD2DarkMageT1_6 = 1014
    DD2DarkMageT1_7 = 1015
    DD2DarkMageT1_8 = 1016
    DD2DarkMageT1_9 = 1017
    DD2DarkMageT1_10 = 1018
    DD2DarkMageT1_11 = 1019
    DD2WitherBeast1 = 1020
    DD2WitherBeast2 = 1021
    DD2WitherBeast3 = 1022
    DD2WitherBeast4 = 1023
    OgreSpit1 = 1024
    OgreSpit2 = 1025
    OgreSpit3 = 1026
    DD2Crystal1 = 1027
    DD2Crystal2 = 1028
    DD2Crystal3 = 1029
    DD2Crystal4 = 1030
    DD2GoblinBomberT1_1 = 1031
    DD2GoblinBomberT1_2 = 1032
    DD2GoblinBomberT1_3 = 1033
    DD2GoblinBomberT2_1 = 1034
    DD2GoblinBomberT2_2 = 1035
    DD2GoblinBomberT2_3 = 1036
    DD2GoblinBomberT3_1 = 1037
    DD2GoblinBomberT3_2 = 1038
    DD2GoblinBomberT3_3 = 1039
    DD2WyvernT2_1 = 1040
    DD2WyvernT2_2 = 1041
    DD2WyvernT2_3 = 1042
    DD2WyvernT2_4 = 1043
    DD2WyvernT2_5 = 1044
    DD2WyvernT3_1 = 1045
    DD2WyvernT3_2 = 1046
    DD2WyvernT3_3 = 1047
    DD2WyvernT3_4 = 1048
    DD2WyvernT3_5 = 1049
    DD2KoboldFlyerT2_1 = 1050
    DD2KoboldFlyerT2_2 = 1051
    DD2KoboldFlyerT2_3 = 1052
    DD2KoboldFlyerT2_4 = 1053
    DD2KoboldFlyerT2_5 = 1054
    DD2JavelinThrowerT2_1 = 1055
    DD2JavelinThrowerT2_2 = 1056
    DD2JavelinThrowerT2_3 = 1057
    DD2JavelinThrowerT2_4 = 1058
    DD2JavelinThrowerT2_5 = 1059
    DD2JavelinThrowerT2_6 = 1060
    DD2JavelinThrowerT3_1 = 1061
    DD2JavelinThrowerT3_2 = 1062
    DD2JavelinThrowerT3_3 = 1063
    DD2JavelinThrowerT3_4 = 1064
    DD2JavelinThrowerT3_5 = 1065
    DD2JavelinThrowerT3_6 = 1066
    DD2GoblinT2_1 = 1067
    DD2GoblinT2_2 = 1068
    DD2GoblinT2_3 = 1069
    DD2GoblinT2_4 = 1070
    DD2GoblinT3_1 = 1071
    DD2GoblinT3_2 = 1072
    DD2GoblinT3_3 = 1073
    DD2GoblinT3_4 = 1074
    DD2DrakinT2_1 = 1075
    DD2DrakinT2_2 = 1076
    DD2DrakinT2_3 = 1077
    DD2DrakinT2_4 = 1078
    DD2BetsyTail = 1079
    DD2BetsyFoot = 1080
    DD2BetsyHead = 1081
    DD2BetsyBody = 1082
    DD2BetsyWing1 = 1083
    DD2BetsyWing2 = 1084
    DD2BetsyHand = 1085
    DD2BetsyButt = 1086
    AmbientFloorCloud1 = 1087
    AmbientFloorCloud2 = 1088
    AmbientFloorCloud3 = 1089
    AmbientFloorCloud4 = 1090
    AmbientAirborneCloud1 = 1091
    AmbientAirborneCloud2 = 1092
    AmbientAirborneCloud3 = 1093
    WalkingAntlion1 = 1094
    WalkingAntlion2 = 1095
    WalkingAntlion3 = 1096
    WalkingAntlion4 = 1097
    FlyingAntlion1 = 1098
    FlyingAntlion2 = 1099
    FlyingAntlion3 = 1100
    FlyingAntlion4 = 1101
    LarvaeAntlion1 = 1102
    LarvaeAntlion2 = 1103
    EyeballFlyingFish1 = 1104
    EyeballFlyingFish2 = 1105
    ZombieMerman1 = 1106
    ZombieMerman2 = 1107
    ZombieMerman3 = 1108
    Golfer1 = 1109
    Golfer2 = 1110
    Golfer3 = 1111
    Golfer4 = 1112
    TreeLeaf_HallowJim = 1113
    TreeLeaf_HallowLastJim = 1121
    DesertPot1 = 1122
    DesertPot2 = 1123
    DesertPot3 = 1124
    Balloon_White_1 = 1125
    Balloon_White_2 = 1126
    Balloon_White_3 = 1127
    Balloon_Red_1 = 1128
    Balloon_Red_2 = 1129
    Balloon_Red_3 = 1130
    Balloon_Orange_1 = 1131
    Balloon_Orange_2 = 1132
    Balloon_Orange_3 = 1133
    Balloon_Yellow_1 = 1134
    Balloon_Yellow_2 = 1135
    Balloon_Yellow_3 = 1136
    Balloon_Green_1 = 1137
    Balloon_Green_2 = 1138
    Balloon_Green_3 = 1139
    Balloon_Blue_1 = 1140
    Balloon_Blue_2 = 1141
    Balloon_Blue_3 = 1142
    Balloon_Purple_1 = 1143
    Balloon_Purple_2 = 1144
    Balloon_Purple_3 = 1145
    Balloon_String = 1146
    WaterDripDesert2 = 1147
    Ladybug1 = 1148
    Ladybug2 = 1149
    Seagull1 = 1150
    Seagull2 = 1151
    Pupfish1 = 1152
    Pupfish2 = 1153
    Grebe1 = 1154
    Grebe2 = 1155
    Grebe3 = 1156
    Rat1 = 1157
    Rat2 = 1158
    Rat3 = 1159
    EbonsandDrip = 1160
    CrimsandDrip = 1161
    PearlsandDrip = 1162
    LittleRat1 = 1163
    Owl1 = 1164
    Owl2 = 1165
    Turtle1 = 1166
    Turtle2 = 1167
    Turtle3 = 1168
    TurtleJungle1 = 1169
    TurtleJungle2 = 1170
    TurtleJungle3 = 1171
    BloodNautilus1 = 1172
    BloodNautilus2 = 1173
    BloodNautilus3 = 1174
    BloodNautilus4 = 1175
    BloodNautilus5 = 1176
    BloodNautilus6 = 1177
    BloodNautilus7 = 1178
    BloodSquid1 = 1179
    BloodSquid2 = 1180
    BloodSquid3 = 1181
    BloodSquid4 = 1182
    GoblinShark1 = 1183
    GoblinShark2 = 1184
    GoblinShark3 = 1185
    GoblinShark4 = 1186
    BloodEel1 = 1187
    BloodEel2 = 1188
    BloodEel3 = 1189
    Gnome1 = 1190
    Gnome2 = 1191
    Gnome3 = 1192
    Gnome4 = 1193
    SeaTurtle1 = 1194
    SeaTurtle2 = 1195
    SeaTurtle3 = 1196
    Seahorse1 = 1197
    Dolphin1 = 1198
    Dolphin2 = 1199
    Dolphin3 = 1200
    ChumBucket = 1201
    FogMachineCloud1 = 1202
    FogMachineCloud2 = 1203
    FogMachineCloud3 = 1204
    Dandelion1 = 1205
    Dandelion2 = 1206
    Dandelion3 = 1207
    ChumBucketFloatingChunks = 1208
    SparkleGuitar = 1209
    RockGolem1 = 1210
    RockGolem2 = 1211
    RockGolem3 = 1212
    RockGolem4 = 1213
    MaggotZombie1 = 1214
    MaggotZombie2 = 1215
    MaggotZombie3 = 1216
    MaggotZombieMaggotPieces = 1217
    ShadowMimicCoins = 1218
    BestiaryGirl1 = 1219
    BestiaryGirl2 = 1220
    BestiaryGirl3 = 1221
    BestiaryGirl4 = 1222
    BestiaryGirlFox1 = 1223
    BestiaryGirlFox2 = 1224
    LightningBunnySparks = 1225
    KingSlimePetCrown = 1226
    GemSquirrelAmethyst = 1227
    GemSquirrelTopaz = 1228
    GemSquirrelSapphire = 1229
    GemSquirrelEmerald = 1230
    GemSquirrelRuby = 1231
    GemSquirrelDiamond = 1232
    GemSquirrelAmber = 1233
    GemBunnyAmethystHead = 1234
    GemBunnyAmethystBack = 1235
    GemBunnyTopazHead = 1236
    GemBunnyTopazBack = 1237
    GemBunnySapphireHead = 1238
    GemBunnySapphireBack = 1239
    GemBunnyEmeraldHead = 1240
    GemBunnyEmeraldBack = 1241
    GemBunnyRubyHead = 1242
    GemBunnyRubyBack = 1243
    GemBunnyDiamondHead = 1244
    GemBunnyDiamondBack = 1245
    GemBunnyAmberHead = 1246
    GemBunnyAmberBack = 1247
    TreeLeaf_VanityTreeSakura = 1248
    TreeLeaf_GemTreeTopaz = 1249
    TreeLeaf_GemTreeAmethyst = 1250
    TreeLeaf_GemTreeSapphire = 1251
    TreeLeaf_GemTreeEmerald = 1252
    TreeLeaf_GemTreeRuby = 1253
    TreeLeaf_GemTreeDiamond = 1254
    TreeLeaf_GemTreeAmber = 1255
    MagmaSnailShell = 1256
    TreeLeaf_VanityTreeYellowWillow = 1257
    QueenSlimeCrown = 1258
    QueenSlimeWing = 1259
    QueenSlimeMinionPurpleWing = 1260
    QueenSlimePetCrown = 1261
    HallowBoss1 = 1262
    HallowBoss2 = 1263
    HallowBoss3 = 1264
    HallowBoss4 = 1265
    HallowBoss5 = 1266
    HallowBoss6 = 1267
    HallowBoss7 = 1268
    PlanteroSombrero = 1269
    DeerclopsHead = 1270
    DeerclopsAntler = 1271
    DeerclopsBody = 1272
    DeerclopsArm = 1273
    DeerclopsLeg = 1274
    Stinkbug1 = 1275
    Stinkbug2 = 1276
    ScarletMacaw1 = 1277
    TreeLeaf_TreeAsh = 1278
    ScarletMacaw2 = 1279
    BlueMacaw1 = 1280
    BlueMacaw2 = 1281
    Toucan1 = 1282
    Toucan2 = 1283
    YellowCockatiel1 = 1284
    YellowCockatiel2 = 1285
    GrayCockatiel1 = 1286
    GrayCockatiel2 = 1287
    ArmsDealerShimmeredHead = 1288
    ArmsDealerShimmeredArm = 1289
    ArmsDealerShimmeredLeg = 1290
    ClothierShimmeredHead = 1291
    ClothierShimmeredArm = 1292
    ClothierShimmeredLeg = 1293
    CyborgShimmeredHead = 1294
    CyborgShimmeredArm = 1295
    CyborgShimmeredLeg = 1296
    DemolitionistShimmeredHead = 1297
    DemolitionistShimmeredHeadParty = 1298
    DemolitionistShimmeredArm = 1299
    DemolitionistShimmeredLeg = 1300
    DryadShimmeredHead = 1301
    DryadShimmeredArm = 1302
    DryadShimmeredLeg = 1303
    DyeTraderShimmeredHead = 1304
    DyeTraderShimmeredHeadParty = 1305
    DyeTraderShimmeredArm = 1306
    DyeTraderShimmeredLeg = 1307
    GoblinTinkererShimmeredHead = 1308
    GoblinTinkererShimmeredArm = 1309
    GoblinTinkererShimmeredLeg = 1310
    GolferShimmeredHead = 1311
    GolferShimmeredArm = 1312
    GolferShimmeredLeg = 1313
    GuideShimmeredHead = 1314
    GuideShimmeredArm = 1315
    GuideShimmeredLeg = 1316
    MechanicShimmeredHead = 1317
    MechanicShimmeredArm = 1318
    MechanicShimmeredLeg = 1319
    MerchantShimmeredHead = 1320
    MerchantShimmeredHeadParty = 1321
    MerchantShimmeredArm = 1322
    MerchantShimmeredLeg = 1323
    NurseShimmeredHead = 1324
    NurseShimmeredHeadParty = 1325
    NurseShimmeredArm = 1326
    NurseShimmeredLeg = 1327
    OldManShimmeredHead = 1328
    OldManShimmeredArm = 1329
    OldManShimmeredLeg = 1330
    PainterShimmeredHead = 1331
    PainterShimmeredLeg = 1332
    PainterShimmeredPaintcan = 1333
    PainterShimmeredPaintbrush = 1334
    PartyGirlShimmeredHead = 1335
    PartyGirlShimmeredArm = 1336
    PartyGirlShimmeredLeg = 1337
    SantaClausShimmeredHead = 1338
    SantaClausShimmeredHeadParty = 1339
    SantaClausShimmeredArm = 1340
    SantaClausShimmeredLeg = 1341
    SkeletonMerchantShimmeredHead = 1342
    SkeletonMerchantShimmeredHeadParty = 1343
    SkeletonMerchantShimmeredArm = 1344
    SkeletonMerchantShimmeredLeg = 1345
    SkeletonMerchantShimmeredBag = 1346
    SkeletonMerchantShimmeredScrap = 1347
    SteampunkerShimmeredHead = 1348
    SteampunkerShimmeredArm = 1349
    SteampunkerShimmeredLegs = 1350
    StylistShimmeredHead = 1351
    StylistShimmeredHair = 1352
    StylistShimmeredArm = 1353
    StylistShimmeredLeg = 1354
    TavernkeepShimmeredHead = 1355
    TavernkeepShimmeredArm = 1356
    TavernkeepShimmeredLeg = 1357
    TaxCollectorShimmeredHead = 1358
    TaxCollectorShimmeredArm = 1359
    TaxCollectorShimmeredLeg = 1360
    TaxCollectorShimmeredCane = 1361
    TravelingMerchantShimmeredHead = 1362
    TravelingMerchantShimmeredHeadParty = 1363
    TravelingMerchantShimmeredArm = 1364
    TravelingMerchantShimmeredLeg = 1365
    TruffleShimmeredHead = 1366
    TruffleShimmeredArm = 1367
    TruffleShimmeredLeg = 1368
    WitchDoctorShimmeredHead = 1369
    WitchDoctorShimmeredArm = 1370
    WitchDoctorShimmeredLeg = 1371
    WitchDoctorShimmeredSpear = 1372
    WizardShimmeredHat = 1373
    WizardShimmeredClothScrap1 = 1374
    WizardShimmeredClothScrap2 = 1375
    WizardShimmeredClothScrap3 = 1376
    BestiaryGirlShimmeredHead = 1377
    BestiaryGirlShimmeredHeadTransformed = 1378
    BestiaryGirlShimmeredArm = 1379
    BestiaryGirlShimmeredArmTransformed = 1380
    BestiaryGirlShimmeredLeg = 1381
    BestiaryGirlShimmeredTail = 1382
    ShimmerDrip = 1383
    HornetHeadHoney = 1384
    HornetHeadSpikey = 1385
    PufferfishSmallHead = 1386
    PufferfishSmallTail = 1387
    PufferfishBigChunk1 = 1388
    PufferfishBigChunk2 = 1389
    PufferfishBigTail = 1390
    MossZombieHead = 1391
    MossZombieChunk1 = 1392
    MossZombieChunk2 = 1393
    PrettyMirror1 = 1394
    PrettyMirror2 = 1395
    PrettyMirror3 = 1396
    PrettyMirror4 = 1397
    RedHatSkeletronHead1 = 1398
    RedHatSkeletronHead2 = 1399
    RedHatSkeletronHead3 = 1400
    RedHatSkeletronHand1 = 1401
    RedHatSkeletronHand2 = 1402
    OrcaHead = 1403
    OrcaTail = 1404
    OrcaChunk1 = 1405
    OrcaChunk2 = 1406
    Count = 1407


class GoreIDSets:
    class Main:
        factory = SetFactory(GoreID.Count)
        SpecialAI = factory.create_int_set(0, 860, 1, 892, 1, 893, 1, 825, 2, 826, 2, 827, 2, 1014, 1, 1015, 1, 1016, 1, 910, 3, 911, 3, 912, 3, 913, 3, 914, 3, 915, 3, 916, 3, 917, 3, 918, 3, 919, 3, 920, 3, 921, 3, 922, 3, 923, 3, 924, 3, 925, 3, 1113, 3, 1114, 3, 1115, 3, 1116, 3, 1117, 3, 1118, 3, 1119, 3, 1120, 3, 1121, 3, 1248, 3, 1249, 3, 1250, 3, 1251, 3, 1252, 3, 1253, 3, 1254, 3, 1255, 3, 1257, 3, 1278, 3, 1007, 3, 1008, 3, 1087, 4, 1088, 4, 1089, 4, 1090, 4, 1091, 5, 1092, 5, 1093, 5, 1202, 6, 1203, 6, 1204, 6, 1225, 7)
        DisappearSpeed = factory.create_int_set(1, 1007, 3, 1008, 10, 1024, 7, 1025, 7, 1026, 7)
        DisappearSpeedAlpha = factory.create_int_set(1, 1007, 3, 1008, 10, 1024, 7, 1025, 7, 1026, 7, 1218, 3)
        IsDrip = factory.create_bool_set(False, 706, 714, 713, 707, 715, 711, 1147, 709, 710, 708, 712, 716, 717, 943, 1160, 1161, 1162, 1383)



from terrex.id import MessageID
from terrex.net.streamer import Writer
from terrex.packet.base import ClientPacket


class CrystalInvasionStart(ClientPacket):
    id = MessageID.CrystalInvasionStart

    def __init__(self, x: int = 0, y: int = 0):
        self.x = x
        self.y = y

    def write(self, writer: Writer):
        writer.write_short(self.x)
        writer.write_short(self.y)



from terrex.id import MessageID
from terrex.net.streamer import Reader, Writer
from terrex.packet.base import SyncPacket


class SyncTilePicking(SyncPacket):
    id = MessageID.SyncTilePicking

    def __init__(self, player_id: int = 0, x: int = 0, y: int = 0, pick_damage: int = 0):
        self.player_id = player_id
        self.x = x
        self.y = y
        self.pick_damage = pick_damage

    def read(self, reader: Reader) -> None:
        self.player_id = reader.read_byte()
        self.x = reader.read_short()
        self.y = reader.read_short()
        self.pick_damage = reader.read_byte()

    def write(self, writer: Writer) -> None:
        writer.write_byte(self.player_id)
        writer.write_short(self.x)
        writer.write_short(self.y)
        writer.write_byte(self.pick_damage)



from terrex.entity.entity import Entity


class NPC(Entity):
    # size
    width = 10
    height = 10

    # ladybug
    lady_bug_good_luck_time: int = 43200
    lady_bug_bad_luck_time: int = -10800
    lady_bug_rain_time: int = 1800
    maximum_amount_of_times_lady_bug_rain_can_stack: int = 10 * lady_bug_rain_time



from dataclasses import dataclass

from terrex.net.streamer import Reader, Writer


@dataclass
class Vec2:
    x: float = 0.0
    y: float = 0.0

    TILE_TO_POS_SCALE: float = 16.0

    @classmethod
    def read(cls, reader: Reader) -> 'Vec2':
        return cls(
            reader.read_single(),
            reader.read_single(),
        )

    def write(self, writer: Writer) -> None:
        writer.write_single(self.x)
        writer.write_single(self.y)

    @classmethod
    def from_tile_pos(cls, x: int, y: int) -> 'Vec2':
        return cls(
            float(x) * cls.TILE_TO_POS_SCALE,
            (float(y) - 2.625) * cls.TILE_TO_POS_SCALE,
        )



from terrex.id import MessageID
from terrex.net.streamer import Reader, Writer

from .base import SyncPacket


class SpecialFX(SyncPacket):
    id = MessageID.SpecialFX

    def __init__(self) -> None:
        self.player_id: int = 0
        self.effect_type: int = (
            0  # 1=SpawnSkeletron, 2=SoundAtPlayer, 3=StartSundialing, 4=BigMimicSpawnSmoke, 5=RegisterTorchGodBestiary
        )

    def read(self, reader: Reader) -> None:
        self.player_id = reader.read_byte()
        self.effect_type = reader.read_byte()

    def write(self, writer: Writer) -> None:
        writer.write_byte(self.player_id)
        writer.write_byte(self.effect_type)



from terrex.id import MessageID
from terrex.net.streamer import Reader, Writer
from terrex.packet.base import SyncPacket


class PlayerBuffs(SyncPacket):
    id = MessageID.PlayerBuffs
    MAX_BUFF = 44  # Player.maxBuffs

    def __init__(self, player_id: int = 0, buffs: list[int] | None = None):
        self.player_id = player_id
        self.buffs = buffs or []

    def read(self, reader: Reader):
        self.player_id = reader.read_byte()
        self.buffs = []
        while len(self.buffs) < self.MAX_BUFF:
            buff = reader.read_ushort()
            if buff == 0:  # packet every end on 0 ushort
                break  # packet ended
            self.buffs.append(buff)

    def write(self, writer: Writer):
        writer.write_byte(self.player_id)
        for b in self.buffs:
            writer.write_ushort(b)
        writer.write_ushort(0)



from dataclasses import dataclass

from terrex.id import LiquidID, TileID
from terrex.net.streamer import Reader
from terrex.net.tile_npc_data import TileNPCData
from terrex.world.world import World

tile_data = TileNPCData()


@dataclass
class Tile:
    type: int = 0  # ushort
    wall: int = 0  # ushort

    # metadata
    liquid: int = 0  # byte
    s_tile_header: int = 0  # ushort
    b_tile_header: int = 0  # byte
    b_tile_header2: int = 0  # byte
    b_tile_header3: int = 0  # byte
    frame_x: int = 0  # short
    frame_y: int = 0  # short

    @classmethod
    def read(cls, reader: Reader, world: World, tile_x: int, tile_y: int) -> tuple["Tile", int]:
        tile = world.tiles.get(tile_x, tile_y)
        if tile is None:
            tile = cls()
            world.tiles.set(tile_x, tile_y, tile)
        else:
            tile.clear_everything()

        # --- flag bytes ---
        b4 = reader.read_byte()

        b = 0
        b2 = 0
        b3 = 0

        has_b = False
        if (b4 & 1) == 1:
            has_b = True
            b = reader.read_byte()

        has_b2 = False
        if has_b and (b & 1) == 1:
            has_b2 = True
            b2 = reader.read_byte()

        if has_b2 and (b2 & 1) == 1:
            b3 = reader.read_byte()

        # --- ACTIVE ---
        was_active = tile.active

        if (b4 & 0x02) == 0x02:
            tile.active = True
            old_tile_type = tile.type

            # print(f"old_tile_type={old_tile_type} x={tile_x} y={tile_y} b4={b4:#04x} b={b:#04x} b2={b2:#04x} b3={b3:#04x} reader_offset={reader.index}")

            if (b4 & 0x20) == 0x20:
                ty_val = reader.read_ushort()
            else:
                ty_val = reader.read_byte()

            tile.type = ty_val

            # print(f"old_tile_type={old_tile_type} ty_val={ty_val}")

            if ty_val > TileID.Count:
                raise ValueError(f"Corrupted tile type {ty_val}")

            if tile_data.tileFrameImportant[ty_val]:
                tile.frame_x = reader.read_short()
                tile.frame_y = reader.read_short()
            elif not was_active or tile.type != old_tile_type:
                tile.frame_x = -1
                tile.frame_y = -1

            if (b2 & 0x08) == 0x08:
                tile.color = reader.read_byte()

        # --- WALL ---
        if (b4 & 0x04) == 0x04:
            tile.wall = reader.read_byte()
            if (b2 & 0x10) == 0x10:
                tile.wall_color = reader.read_byte()

        # --- LIQUID ---
        liquid_code = (b4 & 0x18) >> 3
        if liquid_code != 0:
            tile.liquid = reader.read_byte()

            if (b2 & 0x80) == 0x80:
                tile.shimmer = True
            elif liquid_code > 1:
                if liquid_code == 2:
                    tile.lava = True
                else:
                    tile.honey = True

        # --- b flags ---
        if b > 1:
            if (b & 0x02) == 0x02:
                tile.wire = True
            if (b & 0x04) == 0x04:
                tile.wire2 = True
            if (b & 0x08) == 0x08:
                tile.wire3 = True

            shape = (b & 0x70) >> 4
            if shape != 0 and tile_data.tileSolid[tile.type]:
                if shape == 1:
                    tile.half_brick = True
                else:
                    tile.slope = shape - 1

        # --- b2 flags ---
        if b2 > 1:
            if (b2 & 0x02) == 0x02:
                tile.actuator = True
            if (b2 & 0x04) == 0x04:
                tile.inactive = True
            if (b2 & 0x20) == 0x20:
                tile.wire4 = True
            if (b2 & 0x40) == 0x40:
                high_wall_byte = reader.read_byte()
                tile.wall = (high_wall_byte << 8) | tile.wall

        # --- b3 flags ---
        if b3 > 1:
            if (b3 & 0x02) == 0x02:
                tile.invisible_block = True
            if (b3 & 0x04) == 0x04:
                tile.invisible_wall = True
            if (b3 & 0x08) == 0x08:
                tile.fullbright_block = True
            if (b3 & 0x10) == 0x10:
                tile.fullbright_wall = True

        # --- RLE ---
        rle_code = (b4 & 0xC0) >> 6

        if rle_code == 0:
            rle = 0
        elif rle_code == 1:
            rle = reader.read_byte()
        else:
            rle = reader.read_short()

        return tile, rle

    def copy_from(self, from_tile: 'Tile') -> None:
        self.type = from_tile.type
        self.wall = from_tile.wall
        self.liquid = from_tile.liquid
        self.s_tile_header = from_tile.s_tile_header
        self.b_tile_header = from_tile.b_tile_header
        self.b_tile_header2 = from_tile.b_tile_header2
        self.b_tile_header3 = from_tile.b_tile_header3
        self.frame_x = from_tile.frame_x
        self.frame_y = from_tile.frame_y

    def clear_everything(self) -> None:
        self.type = 0
        self.wall = 0
        self.clear_metadata()

    def clear_metadata(self) -> None:
        self.liquid = 0
        self.s_tile_header = 0
        self.b_tile_header = 0
        self.b_tile_header2 = 0
        self.b_tile_header3 = 0
        self.frame_x = 0
        self.frame_y = 0

    # --- active (бит 5) ---
    @property
    def active(self) -> bool:
        """Возвращает True, если установлен бит 5 (32)"""
        return (self.s_tile_header & 32) == 32

    @active.setter
    def active(self, value: bool) -> None:
        """Устанавливает или сбрасывает бит 5 (32)"""
        if value:
            self.s_tile_header |= 32
        else:
            self.s_tile_header &= 0xFFFF ^ 32  # 65503

    # --- color (биты 0-4) ---
    @property
    def color(self) -> int:
        """Возвращает значение цвета из битов 0–4"""
        return self.s_tile_header & 31  # 0b11111

    @color.setter
    def color(self, value: int) -> None:
        """Устанавливает цвет в биты 0–4, не затрагивая остальные"""
        self.s_tile_header = (self.s_tile_header & 0xFFFF ^ 31) | (value & 31)

    @property
    def wall_color(self) -> int:
        """Биты 0–4 (маска 0b11111 = 31)"""
        return self.b_tile_header & 31

    @wall_color.setter
    def wall_color(self, value: int) -> None:
        # очищаем биты 0–4 (оставляем 5–7), затем вставляем новое значение
        self.b_tile_header = (self.b_tile_header & 0b11100000) | (value & 31)
        self.b_tile_header &= 0xFF  # гарантируем диапазон byte

    @property
    def water(self) -> bool:
        return self.liquid_type == LiquidID.Water

    # --- lava (биты 5–6 = 01) ---
    @property
    def lava(self) -> bool:
        return (self.b_tile_header & 96) == 32

    @lava.setter
    def lava(self, value: bool) -> None:
        if not value:
            self.b_tile_header &= 0b11011111  # 223
        else:
            self.b_tile_header = (self.b_tile_header & 0b10011111) | 32
        self.b_tile_header &= 0xFF

    # --- honey (биты 5–6 = 10) ---
    @property
    def honey(self) -> bool:
        return (self.b_tile_header & 96) == 64

    @honey.setter
    def honey(self, value: bool) -> None:
        if not value:
            self.b_tile_header &= 0b10111111  # 191
        else:
            self.b_tile_header = (self.b_tile_header & 0b10011111) | 64
        self.b_tile_header &= 0xFF

    # --- shimmer (биты 5 и 6 = 96) ---
    @property
    def shimmer(self) -> bool:
        return (self.b_tile_header & 96) == 96

    @shimmer.setter
    def shimmer(self, value: bool) -> None:
        if not value:
            self.b_tile_header &= 0b10011111  # 159
        else:
            self.b_tile_header = (self.b_tile_header & 0b10011111) | 96
        self.b_tile_header &= 0xFF

    # --- liquid_type (0–3) ---
    @property
    def liquid_type(self) -> int:
        return (self.b_tile_header & 96) >> 5

    @liquid_type.setter
    def liquid_type(self, liquid_type: int) -> None:
        if liquid_type == LiquidID.Water:
            self.b_tile_header &= 0b10011111  # 159
        elif liquid_type == LiquidID.Lava:
            self.lava = True
        elif liquid_type == LiquidID.Honey:
            self.honey = True
        elif liquid_type == LiquidID.Shimmer:
            self.shimmer = True

    # skipLiquid (бит 4 b_tile_header3)
    @property
    def skip_liquid(self) -> bool:
        return (self.b_tile_header3 & 16) == 16

    @skip_liquid.setter
    def skip_liquid(self, value: bool) -> None:
        if value:
            self.b_tile_header3 |= 16
        else:
            self.b_tile_header3 &= 0b11101111  # 239
        self.b_tile_header3 &= 0xFF

    # slope (биты 12–14 s_tile_header)
    @property
    def slope(self) -> int:
        return (self.s_tile_header & 0b0111000000000000) >> 12  # 28672

    @slope.setter
    def slope(self, value: int) -> None:
        self.s_tile_header = (self.s_tile_header & 0b1000111111111111) | (
            (value & 7) << 12
        )  # 36863
        self.s_tile_header &= 0xFFFF

    # wallFrameNumber (биты 6–7 b_tile_header2)
    @property
    def wall_frame_number(self) -> int:
        return (self.b_tile_header2 & 0b11000000) >> 6

    @wall_frame_number.setter
    def wall_frame_number(self, value: int) -> None:
        self.b_tile_header2 = (self.b_tile_header2 & 0b00111111) | ((value & 3) << 6)
        self.b_tile_header2 &= 0xFF

    # wallFrameX (биты 0–3 b_tile_header2)
    @property
    def wall_frame_x(self) -> int:
        return (self.b_tile_header2 & 0b00001111) * 36

    @wall_frame_x.setter
    def wall_frame_x(self, value: int) -> None:
        self.b_tile_header2 = (self.b_tile_header2 & 0b11110000) | ((value // 36) & 15)
        self.b_tile_header2 &= 0xFF

    # wallFrameY (биты 0–2 b_tile_header3)
    @property
    def wall_frame_y(self) -> int:
        return (self.b_tile_header3 & 0b00000111) * 36

    @wall_frame_y.setter
    def wall_frame_y(self, value: int) -> None:
        self.b_tile_header3 = (self.b_tile_header3 & 0b11111000) | ((value // 36) & 7)
        self.b_tile_header3 &= 0xFF

    # wires (s_tile_header и b_tile_header)
    @property
    def wire(self) -> bool:
        return (self.s_tile_header & 128) == 128

    @wire.setter
    def wire(self, value: bool) -> None:
        if value:
            self.s_tile_header |= 128
        else:
            self.s_tile_header &= 0b1111111101111111  # 65407
        self.s_tile_header &= 0xFFFF

    @property
    def wire2(self) -> bool:
        return (self.s_tile_header & 256) == 256

    @wire2.setter
    def wire2(self, value: bool) -> None:
        if value:
            self.s_tile_header |= 256
        else:
            self.s_tile_header &= 0b1111111011111111  # 65279
        self.s_tile_header &= 0xFFFF

    @property
    def wire3(self) -> bool:
        return (self.s_tile_header & 512) == 512

    @wire3.setter
    def wire3(self, value: bool) -> None:
        if value:
            self.s_tile_header |= 512
        else:
            self.s_tile_header &= 0b1111110111111111  # 65023
        self.s_tile_header &= 0xFFFF

    @property
    def wire4(self) -> bool:
        return (self.b_tile_header & 128) == 128

    @wire4.setter
    def wire4(self, value: bool) -> None:
        if value:
            self.b_tile_header |= 128
        else:
            self.b_tile_header &= 0b01111111
        self.b_tile_header &= 0xFF

    # frameNumber (биты 4–5 b_tile_header2)
    @property
    def frame_number(self) -> int:
        return (self.b_tile_header2 & 0b00110000) >> 4  # 48

    @frame_number.setter
    def frame_number(self, value: int) -> None:
        self.b_tile_header2 = (self.b_tile_header2 & 0b11001111) | ((value & 3) << 4)  # 207
        self.b_tile_header2 &= 0xFF

    # fullbrightBlock (бит 7 b_tile_header3)
    @property
    def fullbright_block(self) -> bool:
        return (self.b_tile_header3 & 128) == 128

    @fullbright_block.setter
    def fullbright_block(self, value: bool) -> None:
        if value:
            self.b_tile_header3 |= 128
        else:
            self.b_tile_header3 &= 0b01111111  # 127
        self.b_tile_header3 &= 0xFF

    # fullbrightWall (бит 15 s_tile_header)
    @property
    def fullbright_wall(self) -> bool:
        return (self.s_tile_header & 32768) == 32768

    @fullbright_wall.setter
    def fullbright_wall(self, value: bool) -> None:
        if value:
            self.s_tile_header |= 32768
        else:
            self.s_tile_header &= 0b0111111111111111  # 32767
        self.s_tile_header &= 0xFFFF

    # halfBrick (бит 10 s_tile_header)
    @property
    def half_brick(self) -> bool:
        return (self.s_tile_header & 1024) == 1024

    @half_brick.setter
    def half_brick(self, value: bool) -> None:
        if value:
            self.s_tile_header |= 1024
        else:
            self.s_tile_header &= 0b1111101111111111  # 64511
        self.s_tile_header &= 0xFFFF

    # actuator (бит 11 s_tile_header)
    @property
    def actuator(self) -> bool:
        return (self.s_tile_header & 2048) == 2048

    @actuator.setter
    def actuator(self, value: bool) -> None:
        if value:
            self.s_tile_header |= 2048
        else:
            self.s_tile_header &= 0b1111011111111111  # 63487
        self.s_tile_header &= 0xFFFF

    # anyLiquid variants
    @property
    def any_honey(self) -> bool:
        return self.liquid > 0 and self.honey

    @property
    def any_lava(self) -> bool:
        return self.liquid > 0 and self.lava

    @property
    def any_shimmer(self) -> bool:
        return self.liquid > 0 and self.shimmer

    @property
    def any_water(self) -> bool:
        return self.liquid > 0 and self.water

    # blockType
    @property
    def block_type(self) -> int:
        if self.half_brick:
            return 1
        s = self.slope
        if s > 0:
            s += 1
        return s

    # slope helpers
    @property
    def bottom_slope(self) -> bool:
        s = self.slope
        return s == 3 or s == 4

    @property
    def left_slope(self) -> bool:
        s = self.slope
        return s == 2 or s == 4

    @property
    def right_slope(self) -> bool:
        s = self.slope
        return s == 1 or s == 3

    @property
    def top_slope(self) -> bool:
        s = self.slope
        return s == 1 or s == 2

    # checkingLiquid (бит 3 b_tile_header3)
    @property
    def checking_liquid(self) -> bool:
        return (self.b_tile_header3 & 8) == 8

    @checking_liquid.setter
    def checking_liquid(self, value: bool) -> None:
        if value:
            self.b_tile_header3 |= 8
        else:
            self.b_tile_header3 &= 0b11110111  # 247
        self.b_tile_header3 &= 0xFF

    # inActive (бит 6 s_tile_header)
    @property
    def inactive(self) -> bool:
        return (self.s_tile_header & 64) == 64

    @inactive.setter
    def inactive(self, value: bool) -> None:
        if value:
            self.s_tile_header |= 64
        else:
            self.s_tile_header &= 0b1111111110111111  # 65471
        self.s_tile_header &= 0xFFFF

    # invisibleBlock (бит 5 b_tile_header3)
    @property
    def invisible_block(self) -> bool:
        return (self.b_tile_header3 & 32) == 32

    @invisible_block.setter
    def invisible_block(self, value: bool) -> None:
        if value:
            self.b_tile_header3 |= 32
        else:
            self.b_tile_header3 &= 0b11011111  # 223
        self.b_tile_header3 &= 0xFF

    # invisibleWall (бит 6 b_tile_header3)
    @property
    def invisible_wall(self) -> bool:
        return (self.b_tile_header3 & 64) == 64

    @invisible_wall.setter
    def invisible_wall(self, value: bool) -> None:
        if value:
            self.b_tile_header3 |= 64
        else:
            self.b_tile_header3 &= 0b10111111  # 191
        self.b_tile_header3 &= 0xFF

    def __str__(self) -> str:
        return f"Tile(type={self.type}, active={self.active}, wall={self.wall}, slope={self.slope}, fX={self.frame_x}, fY={self.frame_y}"



"""
ProjectileSourceID - autogenerated from decompiled Terraria .cs files
Terraria namespace: Terraria.ID
Terraria version: v1.4.5.5
Generator: TerrariaPyGen v1.1.1 (Maksim Stashkevich)
Generated on: 2026-02-15 09:24
"""

from enum import IntEnum, auto


class ProjectileSourceID(IntEnum):
    NoneValue = 0
    SetBonus_SolarExplosion_WhenTakingDamage = 1
    SetBonus_SolarExplosion_WhenDashing = 2
    SetBonus_ForbiddenStorm = 3
    SetBonus_Titanium = 4
    SetBonus_Orichalcum = 5
    SetBonus_Chlorophyte = 6
    SetBonus_Stardust = 7
    WeaponEnchantment_Confetti = 8
    PlayerDeath_TombStone = 9
    TorchGod = 10
    FallingStar = 11
    PlayerHurt_DropFootball = 12
    SetBonus_GhostHeal = 15
    SetBonus_GhostHurt = 16
    VampireKnives = 18
    BoulderRain = 19
    MeteorRain = 20
    MeteorShower = 21
    StormLightning = 22




"""
MountID - autogenerated from decompiled Terraria .cs files
Terraria namespace: Terraria.ID
Terraria version: v1.4.5.5
Generator: TerrariaPyGen v1.1.1 (Maksim Stashkevich)
Generated on: 2026-02-15 07:03 UTC
"""

from terrex.id.set_factory import SetFactory
from enum import IntEnum, auto


class MountID(IntEnum):
    NoneValue = -1
    Rudolph = 0
    Bunny = 1
    Pigron = 2
    Slime = 3
    Turtle = 4
    Bee = 5
    Minecart = 6
    UFO = 7
    Drill = 8
    Scutlix = 9
    Unicorn = 10
    MinecartMech = 11
    CuteFishron = 12
    MinecartWood = 13
    Basilisk = 14
    DesertMinecart = 15
    FishMinecart = 16
    GolfCartSomebodySaveMe = 17
    BeeMinecart = 18
    LadybugMinecart = 19
    PigronMinecart = 20
    SunflowerMinecart = 21
    HellMinecart = 22
    WitchBroom = 23
    ShroomMinecart = 24
    AmethystMinecart = 25
    TopazMinecart = 26
    SapphireMinecart = 27
    EmeraldMinecart = 28
    RubyMinecart = 29
    DiamondMinecart = 30
    AmberMinecart = 31
    BeetleMinecart = 32
    MeowmereMinecart = 33
    PartyMinecart = 34
    PirateMinecart = 35
    SteampunkMinecart = 36
    Flamingo = 37
    CoffinMinecart = 38
    DiggingMoleMinecart = 39
    PaintedHorse = 40
    MajesticHorse = 41
    DarkHorse = 42
    PogoStick = 43
    PirateShip = 44
    SpookyWood = 45
    Santank = 46
    WallOfFleshGoat = 47
    DarkMageBook = 48
    LavaShark = 49
    QueenSlime = 50
    FartMinecart = 51
    Wolf = 52
    TerraFartMinecart = 53
    Velociraptor = 54
    Rat = 55
    Bat = 56
    RollerSkates = 57
    RollerSkatesGreen = 58
    RollerSkatesWhite = 59
    RollerSkatesPink = 60
    Pixie = 61
    Chillet = 62
    ChilletIgnis = 63
    Count = 64


class MountIDSets:
    class Main:
        factory = SetFactory(MountID.Count)
        CanUseHooks = factory.create_bool_set(False, 54, 57, 58, 59, 60)
        CanDash = factory.create_bool_set(False, 56, 57, 58, 59, 60, 61, 62, 63)
        DoesNotOverrideBodyFrames = factory.create_bool_set(False, 57, 58, 59, 60)
        DoesNotOverrideLegFrames = factory.create_bool_set(False, 57, 58, 59, 60)
        DoesNotOverrideBackpackDraw = factory.create_bool_set(False, 57, 58, 59, 60)
        IsRollerSkates = factory.create_bool_set(False, 57, 58, 59, 60)
        Cart = factory.create_bool_set(False, 6, 13, 11, 15, 16, 18, 19, 20, 21, 22, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 38, 39, 51, 53)
        PlayerIsHidden = factory.create_bool_set(False, 52, 54, 55, 56, 61)
        DontHoldItems = factory.create_bool_set(False, 55, 56, 61)
        DontDismountWhenCCed = factory.create_bool_set(False, 55, 56, 61)



from terrex.id import MessageID
from terrex.localization.network_text import NetworkText
from terrex.net.streamer import Reader
from terrex.packet.base import ServerPacket


class StatusTextSize(ServerPacket):
    id = MessageID.StatusTextSize

    def __init__(self, status_id: int = 0, text: NetworkText | None = None, flags: int = 0):
        self.status_id = status_id
        self.text = text or NetworkText()
        self.flags = flags

    def read(self, reader: Reader):
        self.status_id = reader.read_int()
        self.text = NetworkText.read(reader)
        self.flags = reader.read_byte()



from collections.abc import Callable
from typing import Any, Union, get_args, get_origin


class AnyVar:
    def __init__(
        self,
        default: Any,
        *,
        on_set: Callable[[Any], None] | None = None,
        on_get: Callable[[Any], None] | None = None,
    ):
        self._value = default
        self._on_set = on_set
        self._on_get = on_get

    def __set_name__(self, owner, name):
        self._owner = owner
        self._name = name
        self._var_name = f"_{owner.__name__}__{name}"
        self._expected_type = owner.__annotations__.get(name)
        if self._expected_type is None and self._value is not None:
            self._expected_type = type(self._value)

    def get_field_name(self) -> str:
        if self._name is None:
            raise RuntimeError("Descriptor not bound to a field")
        return f"{self._owner.__name__}.{self._name}"

    def _validate(self, value):
        if self._expected_type is None:
            return

        origin = get_origin(self._expected_type)
        args = get_args(self._expected_type)

        # int, str, MyClass
        if origin is None:
            if not isinstance(value, self._expected_type):
                raise TypeError(
                    f"{self.get_field_name()} must be {self._expected_type.__name__}, "
                    f"got {type(value).__name__}"
                )
            return

        # Union / Optional / int | None
        if origin is Union:
            if not any(isinstance(value, t) for t in args if t is not type(None)):
                if value is None and type(None) in args:
                    return
                allowed = ", ".join(t.__name__ for t in args)
                raise TypeError(
                    f"{self.get_field_name()} must be one of ({allowed}), got {type(value).__name__}"
                )
            return

        # Containers: list[int], set[str], tuple[...]
        if not isinstance(value, origin):
            raise TypeError(
                f"{self.get_field_name()} must be {origin.__name__}, got {type(value).__name__}"
            )

    def __get__(self, instance, owner):
        if instance is None:
            return self
        if not hasattr(instance, self._var_name):
            setattr(instance, self._var_name, self._value)
        value = getattr(instance, self._var_name, self._value)
        if self._on_get:
            self._on_get(value)
        return value

    def __set__(self, instance, value):
        self._validate(value)
        if self._on_set:
            self._on_set(value)
        setattr(instance, self._var_name, value)

    def __int__(self):
        return int(self._value)

    def __float__(self):
        return float(self._value)

    def __str__(self):
        return str(self._value)



from enum import Enum


class ChatCommand(str, Enum):
    SayChat = "Say"
    Roll = "Roll"
    RockPaperScissors = "RPS"
    PVPDeath = "PVPDeath"
    PartyChat = "Party"
    ListPlayers = "Playing"
    Help = "Help"
    Emote = "Emote"
    Emoji = "Emoji"
    Death = "Death"
    BossDamage = "BossDamage"
    AllPVPDeath = "AllPVPDeath"
    AllDeath = "AllDeath"



"""
ID module for Terrex. Contains all Terraria ID enums and constants.
"""

from .AchievementHelperID import AchievementHelperID
from .AmmoID import AmmoID
from .AnimationID import AnimationID
from .ArmorIDs import ArmorIDs
from .BiomeConversionID import BiomeConversionID
from .BuffID import BuffID
from .DustID import DustID
from .ExtrasID import ExtrasID
from .GameEventClearedID import GameEventClearedID
from .GameModeID import GameModeID
from .GameVersionID import GameVersionID
from .GlowMaskID import GlowMaskID
from .GoreID import GoreID
from .HousingCategoryID import HousingCategoryID
from .InvasionID import InvasionID
from .ItemAlternativeFunctionID import ItemAlternativeFunctionID
from .ItemHoldStyleID import ItemHoldStyleID
from .ItemID import ItemID
from .ItemSourceID import ItemSourceID
from .ItemUseStyleID import ItemUseStyleID
from .LiquidID import LiquidID
from .MessageID import MessageID
from .MountID import MountID
from .MusicID import MusicID
from .NPCHeadID import NPCHeadID
from .NPCID import NPCID
from .PaintCoatingID import PaintCoatingID
from .PaintID import PaintID
from .PlayerDifficultyID import PlayerDifficultyID
from .PlayerTeamID import PlayerTeamID
from .PlayerTextureID import PlayerTextureID
from .PlayerVariantID import PlayerVariantID
from .PlayerVoiceID import PlayerVoiceID
from .PlayerVoiceOverrideID import PlayerVoiceOverrideID
from .PrefixID import PrefixID
from .ProjectileDrawLayerID import ProjectileDrawLayerID
from .ProjectileID import ProjectileID
from .ProjectileSourceID import ProjectileSourceID
from .set_factory import SetFactory
from .StatusID import StatusID
from .SurfaceBackgroundID import SurfaceBackgroundID
from .TeleportationSide import TeleportationSide
from .TeleportationStyleID import TeleportationStyleID
from .TileChangeType import TileChangeType
from .TileID import TileID
from .TorchID import TorchID
from .TreeTopID import TreeTopID
from .WallID import WallID
from .WaterStyleID import WaterStyleID


__all__ = [
    "AchievementHelperID",
    "AmmoID",
    "AnimationID",
    "ArmorIDs",
    "BiomeConversionID",
    "BuffID",
    "DustID",
    "ExtrasID",
    "GameEventClearedID",
    "GameModeID",
    "GameVersionID",
    "GlowMaskID",
    "GoreID",
    "HousingCategoryID",
    "InvasionID",
    "ItemAlternativeFunctionID",
    "ItemHoldStyleID",
    "ItemID",
    "ItemSourceID",
    "ItemUseStyleID",
    "LiquidID",
    "MessageID",
    "MountID",
    "MusicID",
    "NPCHeadID",
    "NPCID",
    "PaintCoatingID",
    "PaintID",
    "PlayerDifficultyID",
    "PlayerTeamID",
    "PlayerTextureID",
    "PlayerVariantID",
    "PlayerVoiceID",
    "PlayerVoiceOverrideID",
    "PrefixID",
    "ProjectileDrawLayerID",
    "ProjectileID",
    "ProjectileSourceID",
    "SetFactory",
    "StatusID",
    "SurfaceBackgroundID",
    "TeleportationSide",
    "TeleportationStyleID",
    "TileChangeType",
    "TileID",
    "TorchID",
    "TreeTopID",
    "WallID",
    "WaterStyleID",
]
